// coxe.C Version 7.42e Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-09 08:33:54 tac>
//
// Coxの比例ハザードモデルによる重回帰型生命表解析
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法  :g++ -O -s -W coxe coxe.C -I/usr/local/include
//               -L/usr/local/lib -ldanse -lfune

#include <danse.H>               // dans用ヘッダファイル(fune.H,mate.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法表示用
{
  "coxe  Version 7.42e          Copyleft(Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:coxe [<OPTION> ...] [<DATA FILE> ...]",
  "  The survival time analysis with the Cox's proportional hazards model",
  "  with the first item as the period, the second item as the outcome,",
  "  and the subsequent items as the covariates.",
  "<OPTION>",
  "  -b=<INITIAL VALUE OF BETA-0>[,<INITIAL VALUE OF BETA-1>,...]",
  "    :specify initial values for partial regression coefficients",
  "  -cin=<Cin>:specify criteria for inclusion of covariables(0=<Cin)",
  "  -ct[={-|+}]:specify whether to output a case data",
  "    -:no output  +:output(default)  -cd:-cd=+",
  "  -ct[={-|+}]:specify with or without finite correction",
  "    -:with(default)  +:without  -ct:-ct=+",
  "  -d=<CODE>:specify the code of death(default:-d=0)",
  "  -dnsname[=<FILE>]:specify the file to output DANS data",
  "    default:no output  -dnsname:-dnsname=coxe_dns.d",
  "  -eps[=<ERROR>]:specify the convergence error of the maximum likelihood",
  "    method(0<ERROR)",
  "  -f=<FILE>:reads options from the specified file",
  "  -h[=<MAN COMMAND>]:show online manual(-h:-h=man)",
  "  -ngp={dea|deal|hr|sur|surl}[,...]:select the data for Ngraph",
  "    dea:mortality curve  deal:mortality line  hr:hazard ratio",
  "    sur:survival curve  surl:survival line  default:all",
  "  -ngpname[=<FILE>]:specify the file to output Ngraph data",
  "    default:no output  -ngpname:-ngpname=coxe_ngp.dat",
  "  -pin=<Pin>:specify criteria p-value for inclusion of covariables(0<Pin=<1)",
  "  -res=[-][{def|all|allx|cc[<C-COEFFICIENT>]|cindex|ltab",
  "       |ps[<SURVIVAL RATE>]|selx|stat|v}][,...]:select output results",
  "    -:with -, no output(-res=-:no output all)",
  "    def:same output as default  all:output all",
  "    allx:results of all covariable selection",
  "    cc[<C-COEFFICIENT>]:confidence interval(0=<C-COEFFICIENT<1)",
  "    cindex:C-index  ltab:life tables wih the Kaplan-Meier method",
  "    ps[<SURVIVAL RATE>]:percent survival time",
  "    selx:results of covariable selection  stat:basic statistics",
  "    v:initial values of partial regression coefficients",
  "  -rmax=<NUMBER>:specify the maximum number of repetitions(0=<NUMBER)",
  "  -x<NUMVER>=<VALUE>[,x<NUMVER>=<VALUE>,...]",
  "    :specify the covariate number and value for the Ngraph"
};

const string Title = "=== Cox's Proportional Hazards Model ==="; // 表題
const string DNSNAMEdef = "coxe_dns.d"; // 省略時DANSファイル名
const string NGPNAMEdef = "coxe_ngp.dat"; // 省略時Ngraphファイル名
constexpr double CINdef = 2.0;  // 省略時χ2乗取り込み基準
constexpr double EPSdef = 1.0e-5; // 省略時収束誤差
constexpr double PINdef = 0.1;    // 省略時p値取り込み基準
constexpr double PSdef = 0.5;     // 省略時累積生存率
constexpr int RMAXdef = 200;      // 省略時繰り返し計算最大数

class option                    // オプション集合体
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  lstr blst;                    // 偏回帰係数の初期値リスト
  vecdbl vbvec;                 // 偏回帰係数の初期値ベクトル
  double vcc;                   // 信頼係数
  double vcin;                  // χ2乗d値の取り込み基準
  testtype vct;                 // 有限修正の有無
  string vdcode;                // 死亡のコード
  string vdnsname;              // DANSファイル名
  double veps;                  // 最尤法の収束誤差
  string vngpname;              // Ngraphファイル名
  lstr vngplst;                 // Ngraph出力リスト
  map<string, bool> vngpmap;    // Ngraph出力の有無マップ
  double vpin;                  // P値の取り込み基準
  double vps;                   // 生存時間を推定する累積生存率
  lstr vreslst;                 // 分析結果出力リスト
  map<string, bool> vresmap;    // 分析結果出力の有無マップ
  int vrmax;                    // 最大計算回数
  lstr xlst;                    // 共変数リスト
  vecdbl vxvec;                 // 共変数ベクトル
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  double b(size_t idx) const    // 偏回帰係数初期値を返す
  { return (idx < vbvec.size()) ? vbvec[idx] : BLANKf; }
  double cc() const             // 信頼係数を返す
  { return vcc; }
  double cin() const            // χ2乗値の取り込み基準を返す
  { return vcin; }
  testtype ct() const           // 有限修正の有無を返す
  { return vct; }
  const string& dcode() const   // 死亡のコードを返す
  { return vdcode; }
  const string& dnsname() const // DANSデータファイル名を返す
  { return vdnsname; }
  double eps() const            // 最尤法の収束誤差を返す
  { return veps; }
  const string& ngpname() const // Ngraphデータファイル名を返す
  { return vngpname; }
  bool ngp(const string& code); // Ngraph出力フラッグを返す
  double pin() const            // P値の取り込み基準を返す
  { return vpin; }
  double ps() const             // 生存時間を推定する累積生存率を返す
  { return vps; }
  bool res(const string& code); // 分析結果出力フラッグを返す
  int rmax() const              // 最大計算回数を返す
  { return vrmax; }
  double x(size_t idx) const    // 共変数の値を返す
  { return (idx < vxvec.size()) ? vxvec[idx] : BLANKf; }
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check(size_t ni);         // オプションのチェックとセット
  int set_ngp();                // -ngpオプションのセット
  int set_res();                // -resオプションのセット
};

option::option(const char* c_pname) : vpath(), vpname(), blst(), vbvec(), vcc(CCdef), vcin(CINdef), vct(testcor), vdcode("0"), vdnsname(), veps(EPSdef), vngpname(), vngplst(), vngpmap(), vpin(1.0), vps(PSdef), vreslst(), vresmap(), vrmax(RMAXdef), xlst(), vxvec()
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

bool option::ngp(const string& code) // Ngraph出力フラッグを返す
{
  map<string, bool>::iterator ngppos = vngpmap.find(code);
  if (ngppos == vngpmap.end()) return false; // code無し！
  return vngpmap[code];                     // true/falseを返す
}

bool option::res(const string& code) // 分析結果出力フラッグを返す
{
  map<string, bool>::iterator respos = vresmap.find(code);
  if (respos == vresmap.end()) return false; // code無し！
  return vresmap[code];                     // true/falseを返す
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "b")            // 偏回帰係数初期値の指定
  {
    if (strsplit(blst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "cin")     // χ2乗値の取り込み基準指定
  {
    vcin = opt[1].empty() ? CINdef : strtodf(opt[1]);
    if (vcin < 0.0) err = badopt(vpname, optstr, USAGE, 0.0, DBL_MAX);
  }
  else if (opt[0] == "ct")      // 有限修正の指定
  {
    if (opt[1].empty() || opt[1] == "+") vct = testcor;
    else if (opt[1] == "-") vct = testnon;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "dnsname") // DANSファイル名指定
  {
    if (opt[1].empty()) vdnsname = DNSNAMEdef;
    else vdnsname = opt[1];
    if (vdnsname.size() < 3) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "d")       // 死亡のコード指定
  { vdcode = opt[1]; }
  else if (opt[0] == "eps")     // 最尤法の収束誤差の指定
  {
    veps = opt[1].empty() ? EPSdef : strtodf(opt[1]);
    if (veps < DBL_EPSILON) err = badopt(vpname, optstr, USAGE, DBL_EPSILON, DBL_MAX);
  }
  else if (opt[0] == "ngp")     // Ngraph出力項目指定
  {
    if (strsplit(vngplst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "ngpname")     // Ngraphファイル名指定
  {
    if (opt[1].empty()) vngpname = NGPNAMEdef;
    else vngpname = opt[1];
    if (vngpname.size() < 3) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "pin")     // p値の取り込み基準指定
  {
    vpin = opt[1].empty() ? PINdef : strtodf(opt[1]);
    if (vpin <= 0.0 || 1.0 < vpin) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "res")     // 分析結果の出力指定
  {
    if (strsplit(vreslst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "rmax")    // 最大計算回数指定
  {
    char* ptr = nullptr;
    vrmax = strtol(opt[1].c_str(), &ptr, 0);
    if (vrmax < 0) err = badopt(vpname, optstr, USAGE, 0.0, (double)INT_MAX);
  }
  else if (opt[0][0] == 'x')    // 共変数の値指定
  {
    if (strsplit(xlst, optstr, CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this help message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

int option::check(size_t ni)    // オプションのセットとチェック
{
  if (ni < 3) return disperr(vpname, "At least 3 items are required !");
  if (set_ngp() < 0) return -2; // -resオプションのセット
  if (set_res() < 0) return -3; // -resオプションのセット
  int p = ni - 2;
  if ((int)(blst.size()) > p) return disperr(vpname, "Too many initial value of the partial regression coefficient !");
  vbvec.resize(p);
  vbvec.assign(p, BLANKf);
  if (blst.size() > 0)
  {
    int j = 0;
    for (string bstr : blst)
    {
      double fdata = strtodf(bstr);
      if (fdata == BLANKf) return disperr(vpname, "Wrong initial value->", bstr);
      vbvec[j] = fdata;
      ++j;
    }
  }
  vxvec.resize(p);
  vxvec.assign(p, BLANKf);
  if (xlst.empty()) return 0;
  if ((int)(xlst.size()) > p) return disperr(vpname, "Too many covariates !");
  for (string xstr : xlst)
  {
    string opt[2];
    if (getopt(opt, xstr, OPTdev) > 0)
    {
      char* ptr = nullptr;
      int xno = strtol(opt[0].substr(1).c_str(), &ptr, 0);
      if (xno < 1 || p < xno) return disperr(vpname, "Wrong item number for covariate->", xstr);
      double fdata = strtodf(opt[1]);
      if (fdata == BLANKf) return disperr(vpname, "Wrong value for covariate->", xstr);
      vxvec[xno - 1] = fdata;
    }
  }
  return 0;                     // エラーなし
}

int option::set_ngp()           // -ngpオプションのセット
{
  vngpmap["dea"] = vngpmap["deal"] = vngpmap["hr"] = vngpmap["sur"] = vngpmap["surl"] = true;
  if (vngplst.size() > 0)
  {
    if (vngpname == "") return disperr(vpname, "Requires -ngpname option !");
    vngpmap["dea"] = vngpmap["deal"] = vngpmap["hr"] = vngpmap["sur"] = vngpmap["surl"] = false;
    for (string ngp : vngplst)
    {
      map<string, bool>::iterator ngppos = vngpmap.find(ngp);
      if (ngppos == vngpmap.end())
      { return disperr(vpname, "Wrong -ngp=" + ngp + " option !"); }
      else
      { vngpmap[ngp] = true; }
    }
  }
  return 0;                     // エラーなし
}

int option::set_res()           // -resオプションのセット
{
  vresmap["res"] = vresmap["def"] = vresmap["cc"] = vresmap["selx"] = vresmap["stat"] = true;
  vresmap["allx"] = vresmap["cindex"] = vresmap["ltab"] = vresmap["ps"] = vresmap["v"] = false;
  if (vreslst.size() > 0)
  {
    vresmap["def"] = vresmap["cc"] = vresmap["selx"] = vresmap["stat"] = false;
    for (string res : vreslst)
    {
      map<string, bool>::iterator respos = vresmap.find(res);
      if (res == "-")           // 全て出力せず
      { vresmap["res"] = false; }
      else if (res[0] == '-' && res.size() > 1) // 特定の項目を出力せず
      {
        respos = vresmap.find(res.substr(1));
        if (respos == vresmap.end())
        { return disperr(vpname, "Wrong -res=" + res + " option !"); }
        else
        { vresmap[res.substr(1)] = false; }
      }
      else if (res == "def")    // 省略時出力
      {
        vresmap["def"] = vresmap["cc"] = vresmap["selx"] = vresmap["stat"] = true;
      }
      else if (res == "all")
      {
        vresmap["def"] = vresmap["allx"] = vresmap["cc"] = vresmap["cindex"] = vresmap["ltab"] = vresmap["ps"] = vresmap["selx"] = vresmap["stat"] = vresmap["v"] = true;
      }
      else if (res[0] == 'c' && res[1] == 'c' && res.size() > 2)
      {
        vcc = strtodf(res.substr(2)); // 信頼係数
        if (vcc < 0.0 || 1.0 <= vcc) return disperr(vpname, "Wrong -res=" + res + " option !");
        vresmap["cc"] = true;
      }
      else if (res[0] == 'p' && res[1] == 's' && res.size() > 2)
      {
        vps = strtodf(res.substr(2)); // 累積生存率
        if (vps < 0.0 || 1.0 < vps) return disperr(vpname, "Wrong -res=" + res + " option !");
        vresmap["ps"] = true;
      }
      else if (respos == vresmap.end())
      { return disperr(vpname, "Wrong -res=" + res + " option !"); }
      else
      { vresmap[res] = true; }
    }
  }
  return 0;                     // エラーなし
}

class varclass                  // 変数クラス
{
private:
  int v_p;                       // 共変数の数
  int v_n;                       // 例数
  int v_nd;                      // 死亡の例数
  double v_lamda0;               // 平均瞬間死亡率(平均ハザード)
  listlfm v_lifemlst;            // 生命表リスト
  vector<statv<double>> statvec; // 共変数の基礎統計量ベクトル
  mat<double> v_smat;            // 積和行列
  vecdbl v_b0vec;                // 偏回帰係数初期値ベクトル
  vecdbl v_bvec;                 // 偏回帰係数ベクトル
  vecdbl v_vbvec;                // 偏回帰係数の分散ベクトル
  vecint v_checkvec;             // 変数選択の有無ベクトル
  int v_nr;                      // 繰り返し計算数
  double v_l0;                   // 定数項の尤度
  double v_lb;                   // βの尤度
  bool cal_csqr(const option& opt, vecdbl& cvec, const vecdbl& b0vec) const;
  // 変数選択用χ2乗値を計算する
  int sel_invar(vecint& checkvec, vecdbl& btvec, vecdbl& vbtvec, const vecdbl& cvec) const;
  // 取り込み変数の選択と各種設定
  int sel_outvar(const option& opt, vecint& checkvec, vecdbl& bvec, vecdbl& vbvec, vecdbl& btvec, vecdbl& vbtvec) const;
  // 係数ベクトルの代入と追い出し変数の選択
  bool newton_cox(const option& opt, vecdbl& bvec, vecdbl& vbvec, int& r, double& l0, double& lb, const vecint& checkvec) const;
  // Newton法によるCoxモデルの計算(最尤法)
  bool cal_lgh(const option& opt, vec<double>& gv, mat<double>& hmat, double& lb, const vecint& checkvec, const vec<double>& bv) const;
  // 傾斜ベクトルgとHessian行列Hを計算する
  bool sigma_rt(double& sre, vec<double>& srxv, mat<double>& srxxmat, const vecint& checkvec, const vec<double>& bv, const lifem& lifemi) const;
  // リスク集合rt(st個)について加算する
  int search_st(int& dt, double& sret, vec<double>& srxtv, mat<double>& srxxtmat, vec<double>& ztv, const vecint& checkvec, const vec<double>& bv, listlfm_citr& litr) const;
  // t時期のstとdtをサーチし、集計する
public:
  varclass(size_t ni) : v_p(ni - 2), v_n(0), v_nd(0), v_lamda0(0.0), v_lifemlst(), statvec(ni), v_smat(ni), v_b0vec(ni - 2, 0.0), v_bvec(ni - 2, 0.0), v_vbvec(ni - 2, 0.0), v_checkvec(ni - 1, 0), v_nr(0), v_l0(0.0), v_lb(0.0)
  // コンストラクタ
  {}
  ~varclass()                   // デストラクタ
  {}
  int cal(const option& opt, ldata& datalst);
  // データを集計して生命表リスト等をセットする
  bool stepsel(const option& opt); // 増減法で変数を選択する
  bool allsel(const option& opt);  // 全変数を選択する
  int p() const                 // 共変数の数を返す
  { return v_p; }
  int n() const                 // 例数を返す
  { return v_n; }
  int nd() const                // 死亡例数を返す
  { return v_nd; }
  int nr() const                // 繰り返し計算数を返す
  { return v_nr; }
  int nb() const                // 選択した変数数を返す
  { return v_checkvec[v_p]; }
  int check(size_t idx) const   // 変数選択の有無を返す
  { return (idx < v_checkvec.size()) ? v_checkvec[idx] : 0; }
  double b0(size_t idx) const   // 偏回帰係数初期値を返す
  { return (idx < v_b0vec.size()) ? v_b0vec[idx] : 0.0; }
  double b(size_t idx) const    // 偏回帰係数を返す
  { return (idx < v_bvec.size()) ? v_bvec[idx] : 0.0; }
  double vb(size_t idx) const   // 偏回帰係数の分散を返す
  { return (idx < v_vbvec.size()) ? v_vbvec[idx] : 0.0; }
  double lamda0() const         // 平均瞬間死亡率を返す
  { return v_lamda0; }
  double l0() const             // 定数項の尤度を返す
  { return v_l0; }
  double lb() const             // βの尤度を返す
  { return v_lb; }
  const listlfm& lifemlst() const // 生命表リストを返す
  { return v_lifemlst; }
  int n(size_t idx) const       // 共変数の例数を返す
  { return (idx < statvec.size()) ? (statvec[idx]).n() : 0; }
  double mean(size_t idx) const // 共変数の平均値を返す
  { return (idx < statvec.size()) ? (statvec[idx]).mean() : 0.0; }
  double vr(size_t idx) const   // 共変数の不偏分散を返す
  { return (idx < statvec.size()) ? (statvec[idx]).vr() : 0.0; }
  double sd(size_t idx) const   // 共変数の標準偏差を返す
  { return (idx < statvec.size()) ? (statvec[idx]).sd() : 0.0; }
  const mat<double>& smat() const // 積和行列を返す
  { return v_smat; }
};

bool varclass::cal_csqr(const option& opt, vecdbl& cvec, const vecdbl& b0vec) const
// 変数選択用χ2乗値を計算する
{
  vec<double> bv(b0vec), gv(v_p);
  mat<double> hmat(v_p);
  double lb = 0.0;
  vecint tcheckvec(v_p + 1, 1); // 一時的な選択の有無ベクトル(全て選択)
  tcheckvec[v_p] = v_p;
  if (cal_lgh(opt, gv, hmat, lb, tcheckvec, bv) == false) return false; // 変数選択用χ2乗値計算できず
  hmat.inv();
  bv -= hmat * gv;
  for (int i = 0; i < v_p; ++i) cvec[i] = (hmat(i, i) < 0.0) ? bv(i) * bv(i) / (-hmat(i, i)) : 0.0;
  return true;
}

int varclass::sel_invar(vecint& checkvec, vecdbl& btvec, vecdbl& vbtvec, const vecdbl& cvec) const
// 取り込み変数の選択と各種設定
{
  double cmax = 0.0;
  int imax = -1;
  for (int i = 0; i < v_p; ++i)
  {
    if (checkvec[i] == 0 && cvec[i] > cmax) { cmax = cvec[i];  imax = i; }
  }
  if (imax < 0) return imax;    // 取り込み変数なし
  checkvec[imax] = 1;  checkvec[v_p] += 1;
  btvec.resize(checkvec[v_p]);  vbtvec.resize(checkvec[v_p]);
  btvec.assign(btvec.size(), 0.0);
  return imax;                  // 変数番号を返す
}

int varclass::sel_outvar(const option& opt, vecint& checkvec, vecdbl& bvec, vecdbl& vbvec, vecdbl& btvec, vecdbl& vbtvec) const
// 係数ベクトルの代入と追い出し変数の選択
{
  double cmin = opt.cin();
  ldbl pmax = opt.pin();
  int imin = -1, j = 0;
  bool csel = true;                  // χ2乗値による変数選択
  if (opt.pin() < 1.0) csel = false; // 有意確率pによる変数選択
  for (int i = 0; i < v_p; ++i)
  {
    if (checkvec[i])
    {
      bvec[i] = btvec[j];  vbvec[i] = vbtvec[j];
      double csqr = (vbtvec[j] > 0.0) ? btvec[j] * btvec[j] / vbtvec[j] : 0.0;
      if (csel)
      {
        if (csqr < cmin) { cmin = csqr;  imin = i; }
      }
      else
      {
        ldbl p = ctop(csqr, 1);
        if (p > pmax) { pmax = p;  imin = i; }
      }
      ++j;
    }
  }
  if (imin < 0) return imin;    // 追い出し変数なし
  checkvec[imin] = 0;  checkvec[v_p] -= 1;  bvec[imin] = 0.0;
  btvec.resize(checkvec[v_p]);  vbtvec.resize(checkvec[v_p]);
  btvec.assign(btvec.size(), 0.0);
  return imin;                  // 変数番号を返す
}

bool varclass::newton_cox(const option& opt, vecdbl& bvec, vecdbl& vbvec, int& nr, double& l0, double& lb, const vecint& checkvec) const
// Newton法によるCoxモデルの計算(最尤法)
{
  int pt = checkvec[v_p];
  vec<double> bv(pt), gv(pt);
  mat<double> hmat(pt);
  double lold = 0.0;
  cal_lgh(opt, gv, hmat, l0, checkvec, bv); // bv=0の時の対数尤度を計算する
  for (int i = 0; i < pt; ++i) bv.assign(i, bvec[i]);
  bool cal = true;
  nr = 0;
  while (cal && cal_lgh(opt, gv, hmat, lb, checkvec, bv))
  {
    ++nr;  hmat.inv();
    if (fabs(lb - lold) < opt.eps() || (opt.rmax() > 0 && nr >= opt.rmax()))
    { cal = false; }
    else
    { lold = lb;  bv -= hmat * gv; }
  }
  if (cal) return false;        // Newton法発散
  for (int i = 0; i < pt; ++i) { bvec[i] = bv(i);  vbvec[i] = -hmat(i, i); }
  return true;                  // Newon法成功
}

bool varclass::cal_lgh(const option& opt, vec<double>& gv, mat<double>& hmat, double& lb, const vecint& checkvec, const vec<double>& bv) const
// 傾斜ベクトルgとHessian行列Hを計算する
{
  int pt = checkvec[v_p];
  vec<double> ztv(pt), srxtv(pt), srxv(pt);
  mat<double> srxxtmat(pt), srxxmat(pt);
  gv = 0.0;  hmat = 0.0;  lb = 0.0;
  double sre = 0.0, sret = 0.0;
  listlfm_citr litr = v_lifemlst.begin();
  while (litr != v_lifemlst.end() && sigma_rt(sre, srxv, srxxmat, checkvec, bv, *litr)) ++litr;
  if (litr != v_lifemlst.end()) return false; // gとH計算できず
  int dt = 0, st = 0, nt = v_n;
  bool ok = true;
  vec<double> atv(pt);
  mat<double> aatmat(pt);
  litr = v_lifemlst.begin();
  while (litr != v_lifemlst.end() && ok && (st = search_st(dt, sret, srxtv, srxxtmat, ztv, checkvec, bv, litr)) > 0)
  {
    if (dt > 0)
    {
      if (sre <= 0.0)
      { ok = false; }
      else
      {
        lb += inp(bv, ztv) - double(dt) * log(sre);
        atv = srxv;
        atv *= (double)dt / sre;
        gv += ztv - atv;
        atv = srxv;
        atv *= 1.0 / sre;
        aatmat = srxxmat - srxv * atv.trn();
        double ct = 1.0;
        if (opt.ct() == testcor && dt > 1 && nt > 1) ct = (double)(nt - dt) / (double)(nt - 1);
        aatmat *= ct * (double)dt / sre;
        hmat -= aatmat;
      }
    }
    sre -= sret;  srxv -= srxtv;  srxxmat -= srxxtmat;  nt -= st;
  }
  return ok;
}

bool varclass::sigma_rt(double& sre, vec<double>& srxv, mat<double>& srxxmat, const vecint& checkvec, const vec<double>& bv, const lifem& lifemi) const
// リスク集合rt(st個)について加算する
{
  const double Emax = 308.0;    // 指数最大値
  int pt = checkvec[v_p];
  vec<double> xv(pt);
  int j = 0;
  for (int i = 0; i < v_p; ++i)
  {
    if (checkvec[i])
    {
      xv.assign(j, lifemi.x(i));
      ++j;
    }
  }
  double bx = inp(bv, xv);
  if (isnan(bx) || fabs(bx) >= Emax) return false; // 発散
  double ei = exp(bx);
  vec<double> xtv(xv);
  sre += ei;  xtv *= ei;
  srxv += xtv;
  srxxmat += xv * xtv.trn();
  return true;
}

int varclass::search_st(int& dt, double& sret, vec<double>& srxtv, mat<double>& srxxtmat, vec<double>& ztv, const vecint& checkvec, const vec<double>& bv, listlfm_citr& litr) const
// t時期のstとdtをサーチし、集計する
{
  sret = 0.0;  dt = 0;
  srxtv = 0.0;  srxxtmat = 0.0;  ztv = 0.0;
  int st = 0;
  double ts = litr->t();
  while (litr != v_lifemlst.end() && litr->t() == ts && sigma_rt(sret, srxtv, srxxtmat, checkvec, bv, *litr) > 0)
  {
    if (litr->oc() == ocdeath)
    {
      ++dt;
      int j = 0;
      for (int i = 0; i < v_p; ++i)
      {
        if (checkvec[i])
        {
          ztv.add(j, litr->x(i));
          ++j;
        }
      }
    }
    ++litr;  ++st;
  }
  return st;
}

int varclass::cal(const option& opt, ldata& datalst)
// データを集計して生命表リスト等をセットする
{
  v_nd = 0;
  size_t dim = v_p + 2;
  for (size_t i = 0; i < dim; ++i) statvec[i].clear();
  v_smat = 0.0;
  vecdbl fdatavec(dim);
  fdatavec[0] = 1.0;
  int cno = 1;                  // ケース番号
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr, ++cno)
  {
    double t = ditr->fdata(0);
    bool blank = (t == BLANKf) ? true : false;
    for (int j = 1; blank == false && j <= v_p; ++j)
    {
      fdatavec[j] = ditr->fdata(j + 1);
      if (fdatavec[j] == BLANKf) blank = true;
    }
    if (blank == false)
    {
      vecdbl xvec(v_p);
      statvec[0].sigma(t);
      for (int j = 1; j <= v_p; ++j)
      { statvec[j].sigma(fdatavec[j]);  xvec[j - 1] = fdatavec[j]; }
      octype oc = ocalive;      // 生存
      if (ditr->sdata(1) == opt.dcode()) { oc = ocdeath;  ++v_nd; } // 死亡
      else if (ditr->sdata(1) == BLANKs) oc = ocdropout; // 脱落
      lifem lifemi(cno, t, oc, xvec);
      v_lifemlst.push_back(lifemi); // 生命表リストに追加する
      if (t > 0.0)                // ハザードと重回帰分析用集計
      {
        double lnt = log(t);
        statvec[dim - 1].sigma(lnt);  fdatavec[dim - 1] = lnt;
        for (size_t i = 0; i < dim; ++i)
        {
          double x = fdatavec[i];
          for (size_t j = i; j < dim; ++j) v_smat.add(i, j, x * fdatavec[j]);
        }
      }
    }
  }
  v_n = v_lifemlst.size();      // 例数
  v_lamda0 = (statvec[0].sum() > 0.0) ? (double)v_nd / statvec[0].sum() : 0.0;
  if (v_n < 1) return v_n;      // 生命表データなし
  v_lifemlst.sort();            // 生命表リストをソートする
  int i = 0;
  double s = 1.0;
  for (listlfm_itr litr = v_lifemlst.begin(); litr != v_lifemlst.end(); ++litr, ++i)
  {
    if (litr->oc() == ocdeath) s *= (double)(v_n - i - 1) / (double)(v_n - i);
    litr->s(s);                 // 累積生存率をセットする
  }
  for (size_t i = 0; i < dim - 1; ++i)
  {
    for (size_t j = i + 1; j < dim; ++j) v_smat.assign(j, i, v_smat(i, j));
  }
  v_smat.sweep(0);              // 積和行列に変換する
  return v_n;                   // 例数を返す
}

bool varclass::stepsel(const option& opt) // 増減法で変数を選択する
{
  vecint checkvec(v_checkvec);
  vecdbl bvec(v_bvec), vbvec(v_vbvec);
  vecdbl cvec(v_p, 0.0), btvec(bvec), vbtvec(vbvec);
  double l0 = 0.0, lb = 0.0;
  int nr = 0, nrmax = 0, iin = -1, iout = -1;
  bool newtonok = true;
  do {
    newtonok = cal_csqr(opt, cvec, bvec);
    if (newtonok)
    {
      iin = sel_invar(checkvec, btvec, vbtvec, cvec);
      if (iin >= 0)             // 取り込み変数あり
      {
        iout = -1;
        int i = -1;
        do {
          newtonok = newton_cox(opt, btvec, vbtvec, nr, l0, lb, checkvec);
          if (newtonok)
          {
            if (nr > nrmax) nrmax = nr;
            i = sel_outvar(opt, checkvec, bvec, vbvec, btvec, vbtvec);
            if (i >= 0) iout = i;   // 最後に追い出した変数番号を記憶
          }
        } while (newtonok && i >= 0); // 追い出し変数が無くなるまで続行
      }
    }
  } while (newtonok && iin >= 0 && iin != iout && checkvec[v_p] < v_p);
  v_bvec = bvec;  v_vbvec = vbvec;  v_checkvec = checkvec;
  v_nr = nr;  v_l0 = l0;  v_lb = lb;
  return newtonok;              // Newon法の採否を返す
}

bool varclass::allsel(const option& opt) // 全変数を選択する
{
  for (int i = 1; i <= v_p; ++i) v_smat.sweep(i); // 偏回帰係数計算
  const double Beps = DBL_EPSILON * fmax(5.0, v_smat.maxabse());
  for (int i = 0; i < v_p; ++i)
  {
    v_b0vec[i] = -v_smat(i + 1, v_p + 1);
    if (fabs(v_b0vec[i]) < Beps) v_b0vec[i] = 0.0;
  }
  for (int i = 0; i < v_p; ++i)  // オプション指定による初期値
  {
    if (opt.b(i) != BLANKf) v_b0vec[i] = opt.b(i);
  }
  vecdbl bvec(v_b0vec), vbvec(v_vbvec);
  vecint checkvec(v_p + 1, 1);
  checkvec[v_p] = v_p;
  int nr = 0;
  double l0 = 0.0, lb = 0.0;
  bool newtonok = newton_cox(opt, bvec, vbvec, nr, l0, lb, checkvec);
  v_bvec = bvec;  v_vbvec = vbvec;  v_checkvec = checkvec;
  v_nr = nr;  v_l0 = l0;  v_lb = lb;
  return newtonok;
}

class coxclass                  // coxメインクラス
{
private:
  string fname;                 // ファイル名
  void put_resid(ostream& fout, idd& id) const;
  // 分析結果のタイトルとデータ識別情報を出力する
  void put_ltab(ostream& fout, option& opt, const varclass& var) const;
  // Kaplan-Meier法による生命表を出力する
  void put_mst(ostream& fout, const option& opt, double sp, double sn, double tp, double tn, double tlow, double tup, double tmax) const;
  // %生存期間推定値を出力する
  void put_xmsd(ostream& fout, const option& opt, const idd& id, const varclass& var) const;
  // 共変数の基礎統計量を出力する
  void put_cox(ostream& fout, option& opt, ldata& datalst, bool newtonok, bool stepsel, const varclass& var) const;
  // Coxモデルの解析結果を出力する
  void put_b(ostream& fout, const varclass& var) const;
  // 偏回帰係数を出力する
  void put_bci(ostream& fout, const option& opt, const varclass& var) const;
  // 偏回帰係数の信頼区間を出力する
  void put_cindex(ostream& fout, const option& opt, ldata& datalst, const varclass& var) const;
  // C-indexを出力する
  int cal_cindex(const option& opt, int& nci, double& u, double& ci, ldata& hrlst) const;
  // C-indexを計算する
  void put_dns(const option& opt, idd& id, ldata& datalst, bool stepsel, const varclass& var) const;
  // 対数ハザード比をデータファイル形式で出力する
  void put_dnsid(ofstream& fout, idd& id, bool stepsel) const;
  // DANSデータのタイトルとデータ識別情報を出力する
  void plot_coxngp(option& opt, idd& id, ldata& datalst, bool newtonok, bool stepsel, const varclass& var) const;
  // Ngraph用データ出力
  void plot_ngpid(ofstream& fout, const option& opt, idd& id, bool stepsel, const varclass& var) const;
  // Ngraph用データのタイトルとデータ識別情報を出力する
  void plot_ngpsur(ofstream& fout, const option& opt, const varclass& var) const;
  // Ngraph用に累積生存率曲線を出力する
  void plot_ngpsurl(ofstream& fout, const option& opt, const varclass& var) const;
  // Ngraph用に累積生存率折れ線を出力する
  void plot_ngpdea(ofstream& fout, const option& opt, const varclass& var) const;
  // Ngraph用に累積死亡率曲線を出力する
  void plot_ngpdeal(ofstream& fout, const option& opt, const varclass& var) const;
  // Ngraph用に累積死亡率折れ線を出力する
  void plot_ngphr(ofstream& fout, const option& opt, const varclass& var) const;
  // Ngraph用にHRと信頼区間を出力する
public:
  coxclass() : fname()          // コンストラクタ
  {}
  ~coxclass()                   // デストラクタ
  {}
  int get(const char* c_fname, option& opt, idd& id, ldata& datalst);
  // データを読み込み保存・累積する
  void put(option& opt, idd& id, ldata& datalst) const; // 集計結果を出力する
};

void coxclass::put_resid(ostream& fout, idd& id) const
// 分析結果のタイトルとデータ識別情報を出力する
{
  puttitle(fout, Title);
  fout << "\n";
  id.put_title(fout);
  id.put_stra(fout);
  fout << "PERIOD ITEM   :" << id.item(0) << "\n";
  fout << "OUTCOME ITEM  :" << id.item(1) << "\n";
  for (size_t i = 2; i < id.ni(); ++i)
  { fout << "COVARIATE X" << setw(3) << i - 1 << ":" << id.item(i) << "\n"; }
}

void coxclass::put_ltab(ostream& fout, option& opt, const varclass& var) const
// Kaplan-Meier法による生命表を出力する
{
  constexpr int LENline = 75;   // 区切り線の長さ
  fout << "\n<LIFE TABLE WITH THE KAPLAN-MEIER METHOD>\n";
  fout << "  death code:" << opt.dcode() << "  +:censoring   (+):drop out\n";
  fout << "No. case No. Survival Time(outc)     Sur-n/Obs-n       Survival          SE\n";
  putline(fout, LENline);
  double sigw = 0.0, se = 0.0, sp = 1.0, sn = 1.0, zt = 0.0;
  double tmax = 0.0, tp = 0.0, tn = 0.0, tlow = 0.0, tup = 0.0;
  int n = var.n(), i = 0;
  double z = ptoz(1.0 - opt.cc());
  for (listlfm_citr litr = (var.lifemlst()).begin(); litr != (var.lifemlst()).end(); ++litr, ++i)
  {
    fout << setw(4) << i + 1 << setw(8) << litr->cno() << setw(14) << litr->t();
    if (litr->oc() == ocdeath)
    {
      if (i + 1 < n) sigw += 1.0 / (double)(n - i) / (double)(n - i - 1);
      se = litr->s() * sqrt(sigw);
      fout << setw(16) << n - i - 1 << "/" << left << setw(8) << n - i << right;
      fout << setw(12) << litr->s() << setw(12) << se << "\n";
      zt = (se > 0.0) ? (litr->s() - opt.ps()) / se : 0.0;
      if (zt >= z) tlow = litr->t();
      if (tup == 0.0 && -z >= zt) tup = litr->t();
    }
    else
    {
      fout << left;
      if (litr->oc() == ocalive) fout << setw(10) << " +";
      else fout << setw(10) << "(+)";
      fout << right << "(" << setw(5) << n - i;
      fout << "/" << left << setw(5) << n - i << right;
      fout << ")" << setw(14) << litr->s() << setw(12) << se << "\n";
    }
    if (litr->s() >= opt.ps()) { tp = litr->t();  sp = litr->s(); }
    if (tn == 0.0 && opt.ps() >= litr->s())
    { tn = litr->t();  sn = litr->s(); }
    tmax = litr->t();
  }
  putline(fout, LENline);
  if (opt.res("ps")) put_mst(fout, opt, sp, sn, tp, tn, tlow, tup, tmax);
}

void coxclass::put_mst(ostream& fout, const option& opt, double sp, double sn, double tp, double tn, double tlow, double tup, double tmax) const
// %生存期間推定値を出力する
{
  fout << opt.ps() * 100.0 << "% Survival Time=";
  if (tn == 0.0)
  { fout << tmax << " or more"; }
  else
  {
    if (sp > sn) tp += (tn - tp) * (sp - opt.ps()) / (sp - sn);
    fout << tp;
  }
  fout << "  " << opt.cc() * 100.0 << "%CI:Lower=" << tlow << "  Upper=";
  if (tup == 0.0) fout << tmax << " or more";
  else fout << tup;
  fout << endl;
}

void coxclass::put_xmsd(ostream& fout, const option& opt, const idd& id, const varclass& var) const
// 共変数の基礎統計量を出力する
{
  fout << "\n<BASIC STATISTICS:y=ln(" << id.item(0) << ")>\n";
  constexpr int LENline = 70;
  putline(fout, LENline);
  size_t yidx = var.p() + 1;
  fout << "Y   :";
  putmsd(fout, var.n(yidx), var.mean(yidx), var.vr(yidx));
  for (size_t i = 1; i < yidx; ++i)
  {
    fout << "X" << setw(3) << i << ":";
    putmsd(fout, var.n(i), var.mean(i), var.vr(i));
  }
  putline(fout, LENline);
  fout << "death code=" << opt.dcode() << "  deaths=" << var.nd() << "  censorings=" << var.n() - var.nd() << "  hazard=" << var.lamda0() << endl;
}

void coxclass::put_cox(ostream& fout, option& opt, ldata& datalst, bool newtonok, bool stepsel, const varclass& var) const
// Coxモデルの解析結果を出力する
{
  if (stepsel) fout << "\n<RESULT OF STEPWISE FORWARD SELECTION METHOD>";
  else fout << "\n<RESULT OF ALL COVARIATES>";
  fout << "\nNumber of Newton Method Repetitions:" << var.nr();
  if (newtonok == false) fout << "(NEWTON METHOD DIVERGED !)";
  fout << "\n";
  if (stepsel)
  {
    if (opt.pin() < 1.0)
    { fout << "Inclusion Criteria:p=<" << opt.pin() << "  Outclusion Criteria:p > " << opt.pin(); }
    else
    { fout << "Inclusion Criteria:chi-squared>=" << opt.cin() << "  Outclusion Criteria:chi-squared<" << opt.cin(); }
    fout << "\n";
  }
  fout << "Cox's model(finite correction:";
  if (opt.ct() == testcor) fout << "yes";
  else fout << "no";
  fout << "):S(t)=S0(t)^exp(b0+b1x1+..+bjxj+..)\n";
  fout << "S(t):Survival Function  b0:Intercept  bj:Partial Regression Coefficient\n";
  fout << "S0(t):Baseline Survival Function(when all xj=mean)\n";
  if (opt.res("v"))
  {
    fout << "Initial value:";
    for (int i = 0; i < var.p(); ++i) fout << "b" << i + 1 << "=" << var.b0(i) << " ";
    fout << "\n";
  }
  if (newtonok == false) return;
  put_b(fout, var);
  if (opt.res("cc")) put_bci(fout, opt, var);
  int dfb = var.nb();
  fout << "Log Likelihood:L(model)=" << var.lb() << "  L(b0)=" << var.l0() << "  AIC=" << 2.0 * ((double)dfb - var.lb()) << "\n";
  double csqr = 2.0 * (var.lb() - var.l0());
  ldbl pr = ctop(csqr, dfb);
  fout << "Likelihood Ratio Test:chi-squared=" << csqr << "  DF=" << dfb << "  p=" << pr << pind(pr) << "\n";
  if (opt.res("cindex")) put_cindex(fout, opt, datalst, var);
}

void coxclass::put_b(ostream& fout, const varclass& var) const
// 偏回帰係数を出力する
{
  constexpr int Llen = 79;
  fout << "COVAR         bj      SE(bj)  HR=exp(bj) Standard bj Chi-squared     p-value\n";
  putline(fout, Llen);
  double b0 = 0.0;
  for (int i = 0; i < var.p(); ++i)
  {
    if (var.check(i)) b0 += var.b(i) * var.mean(i + 1);
  }
  fout << "ICPT" << setw(12) << -b0 << "\n";
  for (int i = 0; i < var.p(); ++i)
  {
    if (var.check(i))
    {
      double csqr = (var.vb(i) > 0.0) ? var.b(i) * var.b(i) / var.vb(i) : 0.0;
      ldbl pr = ctop(csqr, 1);
      fout << "X" << setw(3) << i + 1 << setw(12) << var.b(i);
      fout << setw(12) << sqrt(var.vb(i)) << setw(12) << exp(var.b(i));
      fout << setw(12) << var.b(i) * var.sd(i + 1);
      fout << setw(12) << csqr << setw(12) << pr << pind(pr) << "\n";
    }
  }
  putline(fout, Llen);
}

void coxclass::put_bci(ostream& fout, const option& opt, const varclass& var) const
// 偏回帰係数の信頼区間を出力する
{
  double z = ptoz(1.0 - opt.cc());
  fout << "\nCOVAR           bj" << setw(3) << opt.cc() * 100.0;
  fout << "%CI Lower       Upper  HR=exp(bj)" << setw(3) << opt.cc() * 100.0;
  fout << "%CI Lower       Upper\n";
  constexpr int Llen = 78;
  putline(fout, Llen);
  for (int i = 0; i < var.p(); ++i)
  {
    if (var.check(i))
    {
      double ci = z * sqrt(var.vb(i));
      double bl = var.b(i) - ci, bu = var.b(i) + ci;
      fout << "X" << setw(3) << i + 1 << setw(14) << var.b(i);
      fout << setw(12) << bl << setw(12) << bu << setw(12) << exp(var.b(i));
      fout << setw(12) << exp(bl) << setw(12) << exp(bu) << "\n";
    }
  }
  putline(fout, Llen);
}

void coxclass::put_cindex(ostream& fout, const option& opt, ldata& datalst, const varclass& var) const
// C-indexを出力する
{
  ldata hrlst;                  // C-index計算用データリスト
  datac cdata(3);
  vecdbl xvec(var.p());
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr)
  {
    cdata.assign(0, ditr->sdata(0));
    cdata.assign(1, ditr->sdata(1));
    double t = ditr->fdata(0);
    bool blank = (t == BLANKf) ? true : false;
    for (int j = 0; blank == false && j < var.p(); ++j)
    {
      xvec[j] = ditr->fdata(j + 2);
      if (xvec[j] == BLANKf) blank = true;
    }
    if (blank == false)
    {
      double z = 0.0;
      for (int j = 0; j <= var.p(); ++j)
      {
        if (var.check(j)) z += var.b(j) * (xvec[j] - var.mean(j + 1));
      }
      cdata.assign(2, z);
      hrlst.push_back(cdata);
    }
  }
  int nci = 0;
  double u = 0.0, ci = 0.0;
  int n = cal_cindex(opt, nci, u, ci, hrlst);
  fout << "C-index=" << u << "/" << nci << "=" << ci << "  1-C-index=" << 1.0 - ci << endl;
}

int coxclass::cal_cindex(const option& opt, int& nci, double& u, double& ci, ldata& hrlst) const
// C-indexを計算する
{
  nci = 0;  u = 0.0;  ci = 0.0;
  int n = 0;
  for (ldata_itr iitr = hrlst.begin(); iitr != hrlst.end(); ++iitr)
  {
    double ti = iitr->fdata(0), fi = iitr->fdata(2);
    ++n;
    octype oi = ocalive;                               // 生存
    if (iitr->sdata(1) == opt.dcode()) oi = ocdeath;   // 死亡
    ldata_itr jitr = iitr;
    ++jitr;
    while (jitr != hrlst.end())
    {
      if (iitr != jitr)
      {
        double tj = jitr->fdata(0), fj = jitr->fdata(2);
        if (tj != BLANKf && fj != BLANKf)
        {
          octype oj = ocalive;                             // 生存
          if (jitr->sdata(1) == opt.dcode()) oj = ocdeath; // 死亡
          if (oi == ocdeath && oj == ocdeath)
          {
            if (ti < tj)
            {
              ++nci;
              if (fi > fj) u = u + 1.0;
              else if (fi == fj) u = u + 0.5;
            }
            else if (ti > tj)
            {
              ++nci;
              if (fi < fj) u = u + 1.0;
              else if (fi == fj) u = u + 0.5;
            }
          }
          else if (oi == ocdeath && oj == ocalive)
          {
            if (ti <= tj)
            {
              ++nci;
              if (fi > fj) u = u + 1.0;
              else if (fi == fj) u = u + 0.5;
            }
          }
          else if (oi == ocalive && oj == ocdeath)
          {
            if (ti >= tj)
            {
              ++nci;
              if (fi < fj) u = u + 1.0;
              else if (fi == fj) u = u + 0.5;
            }
          }
        }
      }
      ++jitr;
    }
  }
  if (nci > 0) ci = u / (double)nci;
  return n;                     // 評価例数を返す
}

void coxclass::put_dns(const option& opt, idd& id, ldata& datalst, bool stepsel, const varclass& var) const
// 対数ハザード比をデータファイル形式で出力する
{
  ofstream fout;
  if (ofopen(fout, (opt.dnsname()).c_str()) == false) return; // ファイルオープン失敗
  put_dnsid(fout, id, stepsel);
  datac cdata(3);
  vecdbl xvec(var.p());
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr)
  {
    cdata.assign(0, ditr->sdata(0));
    cdata.assign(1, ditr->sdata(1));
    double z = BLANKf, t = ditr->fdata(0);
    bool blank = (t == BLANKf) ? true : false;
    for (int j = 0; blank == false && j < var.p(); ++j)
    {
      xvec[j] = ditr->fdata(j + 2);
      if (xvec[j] == BLANKf) blank = true;
    }
    if (blank == false)
    {
      z = 0.0;
      for (int j = 0; j <= var.p(); ++j)
      {
        if (var.check(j)) z += var.b(j) * (xvec[j] - var.mean(j + 1));
      }
    }
    cdata.assign(2, z);
    cdata.put(fout);
  }
}

void coxclass::put_dnsid(ofstream& fout, idd& id, bool stepsel) const
// DANSデータのタイトルとデータ識別情報を出力する
{
  id.put_title(fout, formtag);
  id.put_stra(fout, formtag);
  idd tid(id.title(), 3);
  tid.assign_item(0, id.item(0));
  tid.assign_item(1, id.item(1));
  if (stepsel) tid.assign_item(2, "Log Hazard Ratio(selected covariates)");
  else tid.assign_item(2, "Log Hazard Ratio(all covariates)");
  tid.put_item(fout, formtag);
}

void coxclass::plot_coxngp(option& opt, idd& id, ldata& datalst, bool newtonok, bool stepsel, const varclass& var) const
// Ngraph用データ出力
{
  ofstream fout;
  if (ofopen(fout, (opt.ngpname()).c_str()) == false) return; // ファイルオープン失敗
  if (newtonok == false)
  {
    plot_ngpid(fout, opt, id, stepsel, var);
    fout << "%NEWTON METHOD DIVERGED !\n";
    return;
  }
  bool delml = false;
  if (opt.ngp("sur"))
  {
    fout << "-s1 -dline -w80 -zy0,1,0.2\n";
    fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
    plot_ngpid(fout, opt, id, stepsel, var);
    plot_ngpsur(fout, opt, var);
    delml = true;
  }
  if (opt.ngp("surl"))
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, opt, id, stepsel, var);
    plot_ngpsurl(fout, opt, var);
  }
  if (opt.ngp("dea"))
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, opt, id, stepsel, var);
    plot_ngpdea(fout, opt, var);
  }
  if (opt.ngp("deal"))
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, opt, id, stepsel, var);
    plot_ngpdeal(fout, opt, var);
  }
  if (opt.ngp("hr"))
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dmark,00 -w80 -o400 -zx0,"<< id.ni() - 1 << ",1 -zy0.1,100,10 -eylog\n";
      fout << "%When Drawing CI:-s1 -derrorbar_y -w80\n";
    }
    plot_ngpid(fout, opt, id, stepsel, var);
    plot_ngphr(fout, opt, var);
  }
}

void coxclass::plot_ngpid(ofstream& fout, const option& opt, idd& id, bool stepsel, const varclass& var) const
// Ngraph用データのタイトルとデータ識別情報を出力する
{
  fout << "%";  puttitle(fout, Title);
  fout << "%\n";
  id.put_title(fout, formngp);
  id.put_stra(fout, formngp);
  fout << "%PERIOD ITEM   :" << id.item(0) << "\n";
  fout << "%OUTCOM ITEM   :" << id.item(1) << "\n";
  for (size_t i = 2; i < id.ni(); ++i)
  {
    if (var.check(i - 2))
    { fout << "%COVARIATE x" << setw(3) << i - 1 << ":" << id.item(i) << "\n"; }
  }
  if (stepsel)
  {
    if (opt.pin() < 1.0) fout << "%Inclusion Criteria:p<=" << opt.pin() << "\n";
    else fout << "%Inclusion Criteria:Wald's Chi-squared>=" << opt.cin() << "\n";
  }
  else
  { fout << "%All Covariates\n"; }
  for (size_t i = 2; i < id.ni(); ++i)
  {
    if (var.check(i - 2))
    {
      if (opt.x(i - 2) != BLANKf)
      { fout << "%Covariate with Specified Value:X" << i - 1 << "=" << opt.x(i - 2) << "\n"; }
    }
  }
}

void coxclass::plot_ngpsur(ofstream& fout, const option& opt, const varclass& var) const
// Ngraph用に累積生存率曲線を出力する
{
  double expx = 0.0;
  for (int i = 0; i < var.p(); ++i)
  {
    if (opt.x(i) != BLANKf && var.check(i))
    {
      expx += var.b(i) * (opt.x(i) - var.mean(i + 1));
    }
  }
  expx = exp(expx);
  fout << "%<CUMULATIVE SURVIVAL CURVE>\n";
  fout << "0 1\n";
  double sc = 0.0, spre = 1.0;
  for (listlfm_citr litr = (var.lifemlst()).begin(); litr != (var.lifemlst()).end(); ++litr)
  {
    fout << litr->t() << " " << spre;
    sc = pow(litr->s(), expx);
    if (litr->oc() == ocdeath) fout << "\n" << litr->t() << " " << sc << "\n";
    else fout << " 0 0.02\n";
    spre = sc;
  }
}

void coxclass::plot_ngpsurl(ofstream& fout, const option& opt, const varclass& var) const
// Ngraph用に累積生存率折れ線を出力する
{
  double expx = 0.0;
  for (int i = 0; i < var.p(); ++i)
  {
    if (opt.x(i) != BLANKf && var.check(i))
    {
      expx += var.b(i) * (opt.x(i) - var.mean(i + 1));
    }
  }
  expx = exp(expx);
  fout << "%<CUMULATIVE SURVIVAL LINE>\n";
  fout << "0 1\n";
  double sc = 0.0, spre = 1.0;
  for (listlfm_citr litr = (var.lifemlst()).begin(); litr != (var.lifemlst()).end(); ++litr)
  {
    fout << litr->t() << " ";
    sc = pow(litr->s(), expx);
    if (litr->oc() == ocdeath) fout << sc << "\n";
    else fout << sc << " 0 0.02\n";
    spre = sc;
  }
}

void coxclass::plot_ngpdea(ofstream& fout, const option& opt, const varclass& var) const
// Ngraph用に累積死亡率曲線を出力する
{
  double expx = 0.0;
  for (int i = 0; i < var.p(); ++i)
  {
    if (opt.x(i) != BLANKf && var.check(i))
    {
      expx += var.b(i) * (opt.x(i) - var.mean(i + 1));
    }
  }
  expx = exp(expx);
  fout << "%<CUMULATIVE MORTALITY CURVE>\n";
  fout << "0 0\n";
  double sc = 0.0, spre = 1.0;
  for (listlfm_citr litr = (var.lifemlst()).begin(); litr != (var.lifemlst()).end(); ++litr)
  {
    fout << litr->t() << " " << 1.0 - spre;
    sc = pow(litr->s(), expx);
    if (litr->oc() == ocdeath) fout << "\n" << litr->t() << " " << 1.0 - sc << "\n";
    else fout << " 0 0.02\n";
    spre = sc;
  }
}

void coxclass::plot_ngpdeal(ofstream& fout, const option& opt, const varclass& var) const
// Ngraph用に累積死亡率折れ線を出力する
{
  double expx = 0.0;
  for (int i = 0; i < var.p(); ++i)
  {
    if (opt.x(i) != BLANKf && var.check(i))
    {
      expx += var.b(i) * (opt.x(i) - var.mean(i + 1));
    }
  }
  expx = exp(expx);
  fout << "%<CUMULATIVE MORTALITY LINE>\n";
  fout << "0 0\n";
  double sc = 0.0, spre = 1.0;
  for (listlfm_citr litr = (var.lifemlst()).begin(); litr != (var.lifemlst()).end(); ++litr)
  {
    fout << litr->t() << " ";
    sc = pow(litr->s(), expx);
    if (litr->oc() == ocdeath) fout << 1.0 - sc << "\n";
    else fout << 1.0 - sc << " 0 0.02\n";
    spre = sc;
  }
}

void coxclass::plot_ngphr(ofstream& fout, const option& opt, const varclass& var) const
// Ngraph用にHRと信頼区間を出力する
{
  fout << "%<HAZARD RATIOS AND CONFIDENCE INTERVALS>\n";
  double z = ptoz(1.0 - opt.cc());
  for (int i = 0; i < var.p(); ++i)
  {
    if (var.check(i))
    {
      double ci = z * sqrt(var.vb(i));
      double hri = exp(var.b(i));
      double hrl = exp(var.b(i) - ci), hru = exp(var.b(i) + ci);
      fout << i + 1 << " " << hri << " " << hru - hri << " " << hrl - hri << "\n";
    }
  }
}

int coxclass::get(const char* c_fname, option& opt, idd& id, ldata& datalst)
// データを読み込み保存・累積する
{
  if (c_fname == nullptr) fname.erase();
  else fname = c_fname;
  int nc = fgetdata(fname, id, datalst);
  if (nc < 0) throw runtime_error("Run-time error !");
  if (opt.check(id.ni()) != 0) throw option_error("Run-time option error !");
  return nc;
}

void coxclass::put(option& opt, idd& id, ldata& datalst) const
// 集計結果を出力する
{
  if (id.nc() < 1)              // データなし
  {
    if (opt.res("res"))
    {
      put_resid(cout, id);
      cout << "\n" << endl;
    }
    return;
  }
  varclass var(id.ni());         // 変数オブジェクト
  if (var.cal(opt, datalst) < 1) return; // 有効なデータなし
  if (opt.res("res"))
  {
    if (opt.res("ltab"))
    {
      put_resid(cout, id);
      put_ltab(cout, opt, var);
      cout << "\n\n";
    }
    put_resid(cout, id);
    if (opt.res("stat"))
    {
      put_xmsd(cout, opt, id, var);
      putmat(cout, "<CORRELATION MATRIX:y1=ln("+id.item(0)+")>", matr, var.n(), var.smat(), var.p(), 1);
    }
    if (var.n() <= var.p())     // 重回帰モデル計算できず
    {
      cout << "\n" << endl;
      return;
    }
  }
  bool newtonok = false, putfile = false;
  if (opt.cin() > 0.0 || (0.0 < opt.pin() && opt.pin() < 1.0)) // 変数選択
  {
    newtonok = var.stepsel(opt);
    if (opt.res("selx")) put_cox(cout, opt, datalst, newtonok, true, var);
    if (newtonok)
    {
      if (opt.dnsname() != "") put_dns(opt, id, datalst, true, var);
      if (opt.ngpname() != "") plot_coxngp(opt, id, datalst, newtonok, true, var);
      putfile = true;
    }
    if (var.nb() >= var.p() && opt.res("allx") == false) // 全変数を選択した時
    {
      if (opt.res("res")) cout << "\n" << endl;
      return;
    }
  }
  newtonok = var.allsel(opt);
  if (opt.res("def") || opt.res("allx")) put_cox(cout, opt, datalst, newtonok, false, var);
  if (opt.res("res")) cout << "\n" << endl;
  if (newtonok && putfile == false) // DANSとNgraphデータを出力していない時
  {
    if (opt.dnsname() != "") put_dns(opt, id, datalst, false, var);
    if (opt.ngpname() != "") plot_coxngp(opt, id, datalst, newtonok, false, var);
  }
}

int main(int argc, char* argv[]) // メインルーチン
{
  coxclass cox;                 // メインクラス変数
  option opt(argv[0]);          // オプション
  idd id;                       // データ識別情報
  ldata datalst;                // データリスト
  try
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理フラッグ
    for (int i = 1; i <= argc; ++i)
    {
      if (i >= argc)
      {
        if (fok) cox.get(nullptr, opt, id, datalst); // 標準入力から読み込む
      }
      else if (*(argv[i]) == CHRopt)
      { opt.set(argv[i] + 1);  fok = true; }
      else
      { cox.get(argv[i], opt, id, datalst);  fok = false; }
    }
    cox.put(opt, id, datalst);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "Unknown error occurred !");
    return 255;
  }
  return 0;
}
