// kme.C Version 7.42e  Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-09 09:34:24 tac>
//
// Kaplan-Meier法による生命表解析を行う。
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法  :g++ -O -s -W kme kme.C
//               -I/usr/local/include -L/usr/local/lib -ldanse -lfune

#include <danse.H>              // dans用ヘッダファイル(fune.H,mate.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法表示用
{
  "kme  Version 7.42e           Copyleft(Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:kme [<OPTION> ...]  [<DATA FILE> …]",
  "  The survival time analysis by the Kaplan-Meier method with the first item",
  "  as the group, the secnd item as the period, the third item as the outcome.",
  "<OPTION>",
  "  -al=<ALPHA>:specify the significace level for power analysis(default:0.05)",
  "  -d=<CODE>:specify the code of death(default:-d=0)",
  "  -eps[=<ERROR>]:specify the convergence error of the ML-method(0<ERROR)",
  "  -f=<FILE>:reads options from the specified file",
  "  -h[=<MAN COMMAND>]:show online manual(-h:-h=man)(-h:-h=man)",
  "  -lambda{1|2}=<HAZARD>:specify the hazard for power analysis(0<HAZARD)",
  "  -ngp={d|de|dl|dw|s|se|sl|sw}[,...]:select the data for Ngraph",
  "    d:mortality curve  de:mortality curve with the exponential model",
  "    dl:mortality line  dw:mortality curve with the Weibull-dis model",
  "    s:survival curve  se:survival curve with the exponential-dis model",
  "    sl:suvival line  sw:survival curve with the Weibull model  default:all",
  "  -ngpname[=<FILE>]:specify the file to output Ngraph data",
  "    default:no output  -ngpname:-ngpname=kme_ngp.dat",
  "  -po=<1-BETA>:specify the power for power analysis(0<1-BETA<1 default:0.8)",
  "  -res=[-][{def|all|cc[<C-COEF>]|cmt[{c|o}]|ex|gemt|gwt[{c|o}]|lrt|ltab",
  "       |m[{c|p}]|mst[<RATE>]|pa|phdt|phrt|rmst[<LAST TIME>]|v",
  "       |wei[{del|dif}]}][,...]:select output results",
  "    -:with -, no output(-res=-:no output all)",
  "    def:same output as default  all:output all",
  "    cc[<C-COEF>]:confidence interval(0=<C-COEF<1 cc:cc0.95)",
  "    cmt[n]:Cox-Mantel test(n:no correction)",
  "    ex:parameters of the exponential distribution model",
  "    gemt:generalized Mantel-extension test",
  "    gwt[{c|n}]:generalized Wilcoxon test(c:correction n:no correction)",
  "    lrt:log-rank test  ltab:lafe table with the Kaplan-Meier method",
  "    m[{c|p}]:multiple comparison(c:Cox-Mantel test  p:PHR-test)",
  "    mst[<RATE>]:percent survival time  pa:power analysis  phdt:PHD-test",
  "    phrt:PHR-test  rmst[<LAST TIME>]:restricted mean survival time",
  "    v:calculation process  wei[{del|dif}]:parameters of the Weibll",
  "    default:auto judgment accoding to data",
  "  -rmax=<NUMBER>:specify the maximum number of repetition(0=<NUMBER)",
  "  -weia[=<a0>]:specify the initial value of a0 for the Weibull",
  "    distribution model(-weia:weia=exp(0.1))"
};

const string Title = "=== Survival Time Analysis ==="; // 表題
const string NGPNAMEdef = "kme_ngp.dat"; // 省略時Ngraphファイル名
constexpr double EPSdef = 1.0e-5; // 収束誤差省略時値
constexpr double MSTdef = 0.5;    // MSTの累積生存率省略時値
constexpr double WEIAdef = exp(0.1); // Weibull分布モデルのa初期値省略時値
constexpr int RMAXdef = 200;         // 最大反復回数省略時値
constexpr size_t NPwei = 2;     // Weibull分布モデルのパラメータ数

enum methodtype                 // 計算方法の種類
{
  methodnon,                    // 計算方法指定なし
  methoddel,                    // 差分法
  methoddif,                    // 微分法
  methodetc                     // その他
};

class option                    // オプションクラス
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  double val;                   // 有意水準
  double vcc;                   // 信頼係数
  testtype vcmt;                // Cox-Mantel検定の種類
  string vdcode;                // 死亡のコード
  double veps;                  // 最尤法の収束誤差
  testtype vgwt;                // 一般化Wilcoxon検定の種類
  double vlambda1;              // 群1のハザード
  double vlambda2;              // 群2のハザード
  double vmst;                  // ％生存時間を推定する累積生存率
  string vngpname;              // Ngraphファイル名
  lstr vngplst;                 // Ngraph出力リスト
  map<string, bool> vngpmap;    // Ngraph出力の有無マップ
  double vpo;                   // 検出力
  lstr vreslst;                 // 分析結果出力リスト
  map<string, bool> vresmap;    // 分析結果出力の有無マップ
  int vrmax;                    // 最大反復回数
  double vrmst;                 // RMSTの最終時点
  methodtype vwei;              // Weibull分布モデルの計算方法の種類
  double vweia;                 // Weibull分布モデルのa初期値
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  double al() const             // 有意水準を返す
  { return val; }
  double cc() const             // 信頼係数を返す
  { return vcc; }
  testtype cmt() const          // Cox-Mantel検定の種類を返す
  { return vcmt; }
  const string& dcode() const   // 死亡のコードを返す
  { return vdcode; }
  double eps() const            // 最尤法の収束誤差を返す
  { return veps; }
  testtype gwt() const          // 一般化Wilcoxon検定の種類を返す
  { return vgwt; }
  double lambda1() const         // 群1のハザードを返す
  { return vlambda1; }
  double lambda2() const         // 群2のハザードを返す
  { return vlambda2; }
  double mst() const            // ％生存時間を推定する累積生存率を返す
  { return vmst; }
  const string& ngpname() const // Ngraphデータファイル名を返す
  { return vngpname; }
  bool ngp(const string& code); // Ngraph出力フラッグを返す
  double po() const             // 検出力を返す
  { return vpo; }
  bool res(const string& code); // 分析結果出力フラッグを返す
  int rmax() const              // 最大反復回数を返す
  { return vrmax; }
  double rmst() const           // RMSTの最終時点を返す
  { return vrmst; }
  methodtype wei() const    // Weibull分布モデルの計算方法の種類を返す
  { return vwei; }
  double weia() const           // Weibull分布モデルのa初期値を返す
  { return vweia; }
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check(size_t ni);         // オプションのチェックとセット
  int set_ngp();                // -ngpオプションのセット
  int set_res();                // -resオプションのセット
};

option::option(const char* c_pname) : vpath(), vpname(), val(ALdef), vcc(CCdef), vcmt(testcor), vdcode("0"), veps(EPSdef), vgwt(testaut), vlambda1(BLANKf), vlambda2(BLANKf), vmst(MSTdef), vngpname(), vngplst(), vngpmap(), vpo(POdef), vreslst(), vresmap(), vrmax(RMAXdef), vrmst(DBL_MAX), vwei(methoddif), vweia(WEIAdef)
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

bool option::ngp(const string& code) // Ngraph出力フラッグを返す
{
  map<string, bool>::iterator ngppos = vngpmap.find(code);
  if (ngppos == vngpmap.end()) return false; // code無し！
  return vngpmap[code];                     // true/falseを返す
}

bool option::res(const string& code) // 分析結果出力フラッグを返す
{
  map<string, bool>::iterator respos = vresmap.find(code);
  if (respos == vresmap.end()) return false; // code無し！
  return vresmap[code];                     // true/falseを返す
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "al")           // 有意水準の指定
  {
    val = opt[1].empty() ? ALdef : strtodf(opt[1]);
    if (val <= 0.0 || 1.0 <= val) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "d")       // 死亡のコード指定
  { vdcode = opt[1]; }
  else if (opt[0] == "eps")     // 最尤法の収束誤差の指定
  {
    veps = opt[1].empty() ? EPSdef : strtodf(opt[1]);
    if (veps < DBL_EPSILON) err = badopt(vpname, optstr, USAGE, DBL_EPSILON, DBL_MAX);
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "lambda1")  // 群1のハザード指定
  {
    vlambda1 = opt[1].empty() ? BLANKf : strtodf(opt[1]);
    if (vlambda1 <= 0.0) err = badopt(vpname, optstr, USAGE, 0.0, DBL_MAX);
  }
  else if (opt[0] == "lambda2")  // 群2のハザード指定
  {
    vlambda2 = opt[1].empty() ? BLANKf : strtodf(opt[1]);
    if (vlambda2 <= 0.0) err = badopt(vpname, optstr, USAGE, 0.0, DBL_MAX);
  }
  else if (opt[0] == "ngp")     // Ngraph出力項目指定
  {
    if (strsplit(vngplst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "ngpname")     // Ngraphファイル名指定
  {
    if (opt[1].empty()) vngpname = NGPNAMEdef;
    else vngpname = opt[1];
    if (vngpname.size() < 3) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "po")      // 検出力の指定
  {
    vpo = opt[1].empty() ? POdef : strtodf(opt[1]);
    if (vpo <= 0.0 || 1.0 <= vpo) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "res")     // 分析結果の出力指定
  {
    if (strsplit(vreslst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "rmax")    // 最大反復回数指定
  {
    char* ptr = nullptr;
    vrmax = strtol(opt[1].c_str(), &ptr, 0);
    if (vrmax < 0) err = badopt(vpname, optstr, USAGE, 0.0, (double)INT_MAX);
  }
  else if (opt[0] == "rmst")    // RMSTの最終時点
  {
    vrmst = opt[1].empty() ? DBL_MAX : strtodf(opt[1]);
    if (vrmst < 0.0) err = badopt(vpname, optstr, USAGE, 0.0);
  }
  else if (opt[0] == "weia")    // Weibull分布モデルのa初期値指定
  {
    vweia = opt[1].empty() ? WEIAdef : strtodf(opt[1]);
    if (veps < DBL_EPSILON) err = badopt(vpname, optstr, USAGE, 0.0, DBL_MAX);
  }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

int option::check(size_t ni)    // オプションのセットとチェック
{
  if (ni != 3) return disperr(vpname, "number of items must be 3 !");
  if (sgndiff(vlambda1, vlambda2) == 0) return disperr(vpname, "lambda1 and lambda2 must not be the same value !");
  if (set_ngp() < 0) return -2; // -ngpオプションのエラー
  if (set_res() < 0) return -3; // -resオプションのエラー
  return 0;                     // エラーなし
}

int option::set_ngp()           // -ngpオプションのセット
{
  vngpmap["d"] = vngpmap["de"] = vngpmap["dl"] = vngpmap["dw"] = vngpmap["s"] = vngpmap["se"] = vngpmap["sl"] = vngpmap["sw"] = true;
  if (vngplst.size() > 0)
  {
    if (vngpname == "") return disperr(vpname, "Requires -ngpname option !");
    vngpmap["d"] = vngpmap["de"] = vngpmap["dl"] = vngpmap["dw"] = vngpmap["s"] = vngpmap["se"] = vngpmap["sl"] = vngpmap["sw"] = false;
    for (string ngp : vngplst)
    {
      map<string, bool>::iterator ngppos = vngpmap.find(ngp);
      if (ngppos == vngpmap.end())
      { return disperr(vpname, "-ngp=" + ngp + " option !"); }
      else
      { vngpmap[ngp] = true; }
    }
  }
  return 0;                     // エラーなし
}

int option::set_res()           // -resオプションのセット
{
  vresmap["res"] = vresmap["def"] = vresmap["cc"] = vresmap["cmt"] = vresmap["ex"] = vresmap["mst"] = vresmap["phrt"] = true;
  vresmap["gemt"] = vresmap["gwt"] = vresmap["lrt"] = vresmap["ltab"] = vresmap["mc"] = vresmap["mp"] = vresmap["pa"] = vresmap["phdt"] = vresmap["rmst"] = vresmap["v"] = vresmap["wei"] = false;
  if (vreslst.size() > 0)
  {
    vresmap["def"] = vresmap["cc"] = vresmap["cmt"] = vresmap["ex"] = vresmap["mst"] = vresmap["phrt"] = false;
    for (string res : vreslst)
    {
      map<string, bool>::iterator respos = vresmap.find(res);
      if (res == "-")           // 全て出力せず
      { vresmap["res"] = false; }
      else if (res[0] == '-' && res.size() > 1) // 特定の項目を出力せず
      {
        respos = vresmap.find(res.substr(1));
        if (respos == vresmap.end())
        { return disperr(vpname, "Wrong -res=" + res + " option !"); }
        else
        { vresmap[res.substr(1)] = false; }
      }
      else if (res == "def")    // 省略時出力
      {
        vresmap["def"] = vresmap["cc"] = vresmap["cmt"] = vresmap["ex"] = vresmap["mst"] = vresmap["phrt"] = true;
      }
      else if (res == "all")
      {
        vresmap["def"] = vresmap["cc"] = vresmap["cmt"] = vresmap["ex"] = vresmap["mst"] = vresmap["phdt"] = vresmap["phrt"] = vresmap["gemt"] = vresmap["gwt"] = vresmap["lrt"] = vresmap["ltab"] = vresmap["mc"] = vresmap["mp"] = vresmap["pa"] = vresmap["rmst"] = vresmap["v"] = vresmap["wei"] = true;
      }
      else if (res[0] == 'c' && res[1] == 'c' && res.size() > 2)
      {
        vcc = strtodf(res.substr(2)); // 信頼係数
        if (vcc < 0.0 || 1.0 <= vcc) return disperr(vpname, "Wrong -res=" + res + " option !");
        vresmap["cc"] = true;
      }
      else if (res[0] == 'm' && res[1] == 's' && res[2] == 't' && res.size() > 3)
      {
        vmst = strtodf(res.substr(3)); // 累積生存率
        if (vmst <= 0.0 || 1.0 < vmst) return disperr(vpname, "Wrong -res=" + res + " option !");
        vresmap["mst"] = true;
      }
      else if (res[0] == 'r' && res[1] == 'm' && res[2] == 's' && res[3] == 't' && res.size() > 4)
      {
        vrmst = strtodf(res.substr(4)); // RMST最終時点
        if (vrmst <= 0.0) return disperr(vpname, "Wrong -res=" + res + " option !");
        vresmap["rmst"] = true;
      }
      else if (res == "cmtc")
      { vresmap["cmt"] = true;  vcmt = testcor; }
      else if (res == "cmto")
      { vresmap["cmt"] = true;  vcmt = testorg; }
      else if (res == "gwtc")
      { vresmap["gwt"] = true;  vgwt = testcor; }
      else if (res == "gwto")
      { vresmap["gwt"] = true;  vgwt = testorg; }
      else if (res == "weidel") // 差分法
      { vresmap["wei"] = true;  vwei = methoddel; }
      else if (res == "weidif") // 微分法
      { vresmap["wei"] = true;  vwei = methoddif; }
      else if (respos == vresmap.end())
      { return disperr(vpname, "Wrong -res=" + res + " option !"); }
      else
      { vresmap[res] = true; }
    }
  }
  return 0;                     // エラーなし
}

class statlf : public statv<double> // 基礎統計量+生命表統計量クラス
{
private:
  double vmin;                  // 最小値
  double vmax;                  // 最大値
  int vnd;                      // 死亡例数
public:
  statlf() : statv(), vmin(DBL_MAX), vmax(-DBL_MAX), vnd(0)
  // デフォールトコンストラクタ
  {}
  statlf(double x, int nd) : statv(x), vmin(x), vmax(x), vnd(nd)
  // コンストラクタ
  {}
  statlf(const statlf& statlf0) : statv(statlf0.n(), statlf0.sum(), statlf0.sum2()), vmin(statlf0.vmin), vmax(statlf0.vmax), vnd(statlf0.vnd)
  // コピーコンストラクタ
  {}
  virtual ~statlf()              // デストラクタ
  {}
  const statlf& operator=(const statlf& statlf0); // コピー代入演算子
  void clearl()                                // データをクリアする
  { clear();  vmin = DBL_MAX;  vmax = -DBL_MAX;  vnd = 0; }
  int sigma(double x, int nd);  // データの累積集計
  double min() const            // 最小値を返す
  { return vmin; }
  double max() const            // 最大値を返す
  { return vmax; }
  int nd() const                // 死亡例数を返す
  { return vnd; }
};

const statlf& statlf::operator=(const statlf& statlf0) // コピー代入演算子
{
  if (&statlf0 == this) return *this; // 自分自身への代入を防ぐ
  vn = statlf0.vn;  sigmax = statlf0.sigmax;  sigmax2 = statlf0.sigmax2;
  vmin = statlf0.vmin;  vmax = statlf0.vmax;  vnd = statlf0.vnd;
  return *this;
}

int statlf::sigma(double x, int nd) // データの累積集計
{
  if (x == BLANKf) return 0;    // 累積せず
  ++vn;  sigmax += x;  sigmax2 += x * x;  vnd += nd;
  if (x < vmin) vmin = x;
  if (x > vmax) vmax = x;
  return 1;                     // 累積
}

using mapg = map<string, statlf>; // 群別統計量マップの別名定義
using mapg_itr = map<string, statlf>::iterator; // マップ指定子の別名定義

class varclass                  // 変数クラス
{
private:
  listlf vlifelst;              // 生命表リスト
  int vng;                      // 群数
  mapg vgmap;                   // 群別統計量マップ
  vector<mapg_itr> vgitrvec;    // 群別統計量指定子ベクトル
  bool vzwcor;                  // 一般化Wilcoxon検定の修正フラッグ
  bool vcmcor;                  // Cox-Mantel検定の修正フラッグ
  int vdflr;                    // ログランク検定の自由度
  double vcsqrlr;               // ログランク検定のχ2乗値
  int vdfgem;                   // 一般化拡張Mantel検定の自由度
  double vcsqrgem;              // 一般化拡張Mantel検定のχ2乗値
  int vdfgemr;                  // 一般化拡張Mantel検定交互作用の自由度
  double vcsqrgemr;             // 一般化拡張Mantel検定交互作用のχ2乗値
  double vzw;                   // 一般化Wilcoxon検定のz値
  double v_w;                   // 一般化Wilcoxon検定のW値
  double v_vw;                  // 一般化Wilcoxon検定のW値の分散
  double vcsqrcm;               // Cox-Mantel検定のχ2乗値
  int vdfcmr;                   // Cox-Mantel検定交互作用の自由度
  double vcsqrcmr;              // Cox-Mantel検定交互作用のχ2乗値
  double vb;                    // Coxのβ
  double vseb;                  // Coxのβの標準誤差
  int vnr;                      // Weibull分布モデルの繰り返し計算数
  double vll;                   // Weibull分布モデルの対数尤度
  vec<double> v_hv0;    // Weibull分布モデルのパラメータ初期値ベクトル
  vec<double> v_vhv0; // Weibull分布モデルのパラメータ分散初期値ベクトル
  vec<double> v_hv;   // Weibull分布モデルのパラメータベクトル
  vec<double> v_vhv;  // Weibull分布モデルのパラメータ分散ベクトル
  void cal_km(const option& opt); // 累積生存率と検定の計算
  int search_gcode(const string& code) const;
  // 群コードをサーチしてインデックスを返す
  int log_rank(double& csqr) const; // ログランク検定
  int gem_test(double& cmm, double& cmr) const; // 一般化拡張Mantel検定
  bool g_wilcox(const option& opt, double& zw, double& w, double& vw) const;
  // 一般化Wilcoxon2標本検定
  bool cox_man(const option& opt, double& csqrcm, double& csqrcmr, int& dfcmr, double& b, double& seb, const listlf& lifelst0, const vector<mapg_itr>& gitrvec0) const;
  // Cox-Mantel検定
  bool caldif_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv, int idx) const;
  // 傾斜ベクトルgとHessian行列Hを微分法で計算する
  bool caldel_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv, int idx) const;
  // 傾斜ベクトルgとHessian行列Hを差分法で計算する
  double wei_llfun(const vec<double>& hv, int idx) const;
  // Weibull分布モデルの対数尤度を返す
public:
  varclass(size_t np0) : vlifelst(), vng(0), vgmap(), vgitrvec(), vzwcor(false), vcmcor(false), vdflr(0), vcsqrlr(0.0), vdfgem(0), vcsqrgem(0.0), vdfgemr(0), vcsqrgemr(0.0), vzw(0.0), v_w(0.0), v_vw(0.0), vcsqrcm(0.0), vdfcmr(0), vcsqrcmr(0.0), vb(0.0), vseb(0.0), vnr(0), vll(0.0), v_hv0(np0), v_vhv0(np0), v_hv(np0), v_vhv(np0)
  // コンストラクタ
  {}
  ~varclass()                   // デストラクタ
  {}
  int cal(const option& opt, ldata& datalst);
  // データを集計して生命表リスト等をセットする
  bool newtonwei(const option& opt, int idx, double lambda);
  // Weibull分布モデルをNewton法で計算する
  bool cmtmcom(const option& opt, int iidx, int jidx);
  // Cox-Mantelの多重比較を行う
  const listlf& lifelst() const  // 生命表リストを返す
  { return vlifelst; }
  int ng() const                // 群数を返す
  { return vng; }
  const string& gcode(size_t idx) const // 群コードを返す
  { return (idx < vgitrvec.size()) ? (vgitrvec[idx])->first : BLANKs; }
  int n(size_t idx) const       // 群の例数を返す
  { return (idx < vgitrvec.size()) ? (vgitrvec[idx])->second.n() : 0; }
  int nd(size_t idx) const      // 群の死亡例数を返す
  { return (idx < vgitrvec.size()) ? (vgitrvec[idx])->second.nd() : 0; }
  double sum(size_t idx) const  // 群の観察期間合計を返す
  { return (idx < vgitrvec.size()) ? (vgitrvec[idx])->second.sum() : 0.0; }
  double max(size_t idx) const  // 群の観察期間最大値を返す
  { return (idx < vgitrvec.size()) ? (vgitrvec[idx])->second.max() : 0.0; }
  bool zwcor() const        // 一般化Wilcoxon検定の修正フラッグを返す
  { return vzwcor; }
  bool cmcor() const        // Cox-Mantel検定の修正フラッグを返す
  { return vcmcor; }
  int dflr() const          // ログランク検定の自由度を返す
  { return vdflr; }
  double csqrlr() const     // ログランク検定のχ2乗値を返す
  { return vcsqrlr; }
  int dfgem() const         // 一般化拡張Mantel検定の自由度を返す
  { return vdfgem; }
  double csqrgem() const    // 一般化拡張Mantel検定のχ2乗値を返す
  { return vcsqrgem; }
  int dfgemr() const     // 一般化拡張Mantel検定交互作用の自由度を返す
  { return vdfgemr; }
  double csqrgemr() const // 一般化拡張Mantel検定交互作用のχ2乗値を返す
  { return vcsqrgemr; }
  double zw() const         // 一般化Wilcoxon検定のz値を返す
  { return vzw; }
  double w() const              // 一般化Wilcoxon検定のW値を返す
  { return v_w; }
  double vw() const         // 一般化Wilcoxon検定のW値の分散を返す
  { return v_vw; }
  double csqrcm() const     // Cox-Mantel検定のχ2乗値を返す
  { return vcsqrcm; }
  int dfcmr() const         // Cox-Mantel検定交互作用の自由度を返す
  { return vdfcmr; }
  double csqrcmr() const    // Cox-Mantel検定交互作用のχ2乗値を返す
  { return vcsqrcmr; }
  double b() const          // Coxのβを返す
  { return vb; }
  double seb() const        // Coxのβの標準誤差を返す
  { return vseb; }
  int nr() const            // Weibull分布モデルの繰り返し計算数を返す
  { return vnr; }
  double ll() const         // Weibull分布モデルの対数尤度を返す
  { return vll; }
  double h0(size_t idx) const // Weibull分布モデルのパラメータ初期値を返す
  { return (idx < v_hv0.dim()) ? v_hv0(idx) : 0.0; }
  double vh0(size_t idx) const // Weibull分布モデルのパラメータ分散初期値を返す
  { return (idx < v_vhv0.dim()) ? v_vhv0(idx) : 0.0; }
  double h(size_t idx) const // Weibull分布モデルのパラメータを返す
  { return (idx < v_hv.dim()) ? v_hv(idx) : 0.0; }
  double vh(size_t idx) const // Weibull分布モデルのパラメータ分散を返す
  { return (idx < v_vhv.dim()) ? v_vhv(idx) : 0.0; }
};

void varclass::cal_km(const option& opt) // 累積生存率と検定の計算
{
  vlifelst.sort();              // 生命表リストをソートする
  vecdbl svec(vng, 1.0);
  vecint nvec(vng, 0);
  for (listlf_itr litr = vlifelst.begin(); litr != vlifelst.end(); ++litr)
  {
    int i = search_gcode(litr->code());
    litr->cidx(i);              // 群インデックスをセットする
    int ni = (vgitrvec[i])->second.n();
    if (litr->oc() == ocdeath) svec[i] *= (double)(ni - nvec[i] - 1) / (double)(ni - nvec[i]);
    litr->s(svec[i]);           // 累積生存率をセットする
    ++(nvec[i]);
  }
  vcsqrlr = vcsqrgem = vcsqrgemr = vzw = vcsqrcm = vcsqrcmr = vb = vseb = 0.0;
  vdflr = vdfgem = vdfgemr = vdfcmr = 0;
  if (vng < 2) return;          // 検定できず
  vdflr = log_rank(vcsqrlr);
  vdfgemr = gem_test(vcsqrgem, vcsqrgemr);
  vdfgem = vng - 1;
  if (vdflr == 1)
  {
    vzwcor = g_wilcox(opt, vzw, v_w, v_vw);
    vcmcor = cox_man(opt, vcsqrcm, vcsqrcmr, vdfcmr, vb, vseb, vlifelst, vgitrvec);
  }
}

int varclass::search_gcode(const string& code) const
// 群コードをサーチしてインデックスを返す
{
  bool hit = false;
  size_t i = 0;
  while (hit == false && i < vgitrvec.size())
  {
    if (code == (vgitrvec[i])->first) hit = true; // 発見
    else ++i;
  }
  return hit ? i : -1;          // 群インデックスを返す
}

int varclass::log_rank(double& csqr) const // ログランク検定
{
  vecint dmvec(vng, 0), smvec(vng, 0), wmvec(vng, 0);
  vecdbl devec(vng, 0.0);
  for (int i = 0; i < vng; i++) smvec[i] = vgitrvec[i]->second.n();
  listlf_citr litr = vlifelst.begin();
  double to = litr->t();
  int j = 0, nl = vlifelst.size();
  while (litr != vlifelst.end())
  {
    int s = nl - j, d = 0;
    wmvec.assign(vng, 0);
    while (litr != vlifelst.end() && litr->t() == to)
    {
      int i = litr->cidx();
      if (litr->oc() == ocdeath) { ++d;  ++(dmvec[i]); }
      ++(wmvec[i]);  ++litr;  ++j;
    }
    for (int i = 0; i < vng; ++i)
    {
      devec[i] += (double)d * smvec[i] / s;
      smvec[i] -= wmvec[i];
    }
    if (litr != vlifelst.end()) to = litr->t();
  }
  int df = 0;
  csqr = 0.0;
  for (int i = 0; i < vng; ++i)
  {
    if (devec[i] > 0.0)
    {
      double delta = (double)(dmvec[i]) - devec[i];
      csqr += delta * delta / devec[i];
      ++df;
    }
  }
  return df - 1;                // 自由度を返す
}

int varclass::gem_test(double& cmm, double& cmr) const
// 一般化拡張Mantel検定
{
  vecint dveck(vng, 0), sveck(vng, 0), dwveck(vng, 0);
  for (int i = 0; i < vng; i++) sveck[i] = vgitrvec[i]->second.n();
  vec<double> uevk(vng - 1), uev(vng - 1);
  mat<double> wmatk(vng - 1), wmat(vng - 1);
  double cmt = 0.0;
  int b = 0;
  listlf_citr litr = vlifelst.begin();
  double to = litr->t();
  int nl = vlifelst.size();
  while (litr != vlifelst.end())
  {
    int nd = 0;
    dveck.assign(vng, 0);  dwveck.assign(vng, 0);
    while (litr != vlifelst.end() && litr->t() == to)
    {
      int i = litr->cidx();
      if (litr->oc() == ocdeath) { ++nd;  ++(dveck[i]); }
      ++(dwveck[i]);  ++litr;
    }
    if (nl > 1 && nd > 0)       // 死亡例がある時だけ計算
    {
      double nk = (double)nl, nk0 = (double)nd, nk1 = (double)(nl - nd);
      double rk0 = (nk0 + 1.0) * 0.5;
      double rk1 = nk0 + (nk1 + 1.0) * 0.5;
      double rkm = (rk0 * nk0 + rk1 * nk1) / nk;
      double vkr = (nk0 * rk0 * rk0 + nk1 * rk1 * rk1 - nk * rkm * rkm) / nk;
      bool nok = true;
      for (int i = 0; i < vng - 1; ++i)
      {
        if (sveck[i] < 1) nok = false;
        double nki0 = (double)(dveck[i]), nki = (double)(sveck[i]);
        double nki1 = nki - nki0;
        double di = rk0 * (nki0 - nki * nk0 / nk) + rk1 * (nki1 - nki * nk1 / nk);
        uevk.assign(i, di);
        for (int j = 0; j < vng - 1; ++j)
        {
          if (j == i) wmatk.assign(i, j, nki * (nk - nki) / (nk - 1.0) * vkr);
          else wmatk.assign(i, j, -nki * (double)(sveck[j]) / (nk - 1.0) * vkr);
        }
      }
      if (nok && sveck[vng - 1] > 0) // 全ての群がある時だけ計算
      {
        uev += uevk;  wmat += wmatk;
        wmatk.inv();
        vec<double> tvec(wmatk * uevk);
        double cmk = inp(uevk, tvec);
        cmt += cmk;  ++b;
      }
    }
    if (litr != vlifelst.end())
    {
      for (int i = 0; i < vng; ++i) { sveck[i] -= dwveck[i];  nl -= dwveck[i]; }
      to = litr->t();
    }
  }
  wmat.inv();
  vec tv(wmat * uev);
  cmm = inp(uev, tv);
  cmr = cmt - cmm;
  return (vng - 1) * (b - 1);   // 異質性の検定の自由度を返す
}

bool varclass::g_wilcox(const option& opt, double& zw, double& w, double& vw) const
// 一般化Wilcoxon2標本検定
{
  int nl = (int)(vlifelst.size());
  vecint dd1vec(nl, 0), dw1vec(nl, 0), ddtvec(nl, 0), dwtvec(nl, 0);
  listlf_citr litr = vlifelst.begin();
  double to = litr->t();
  int nt = 0;
  while (litr != vlifelst.end())
  {
    while (litr != vlifelst.end() && litr->t() == to)
    {
      if (litr->oc() == ocdeath)
      {
        ++(ddtvec[nt]);
        if (litr->cidx() == 0) ++(dd1vec[nt]);
      }
      else
      {
        ++(dwtvec[nt]);
        if (litr->cidx() == 0) ++(dw1vec[nt]);
      }
      ++litr;
    }
    ++nt;
    if (litr != vlifelst.end()) to = litr->t();
  }
  zw = 0.0; w = 0.0;  vw = 0.0;
  for (int i = 0; i < nt; ++i)
  {
    int wfix = 0, wflx = 0;
    for (int j = 0; j < nt; ++j)
    {
      if (j < i) { wfix += ddtvec[j];  wflx += ddtvec[j]; }
      else if (j == i) { wfix -= dwtvec[j];  wflx += ddtvec[j]; }
      else wfix -= ddtvec[j] + dwtvec[j];
    }
    w += (double)(dd1vec[i]) * (double)wfix + (double)(dw1vec[i]) * (double)wflx;
    vw += (double)(ddtvec[i]) * (double)wfix * (double)wfix + (double)(dwtvec[i]) * (double)wflx * (double)wflx;
  }
  int n0 = (vgitrvec[0])->second.n();
  vw *= (double)n0 / (double)nl * (double)(nl - n0) / (double)(nl - 1);
  zw = (vw > 0.0) ? (w - sgn(w)) / sqrt(vw) : 0.0;
  if (opt.gwt() == testcor) return true; // 連続修正有
  bool zwcor = true;
  if (opt.gwt() == testorg || (opt.gwt() == testaut && fabs(zw) >= 2.0))
  { zw = w / sqrt(vw);  zwcor = false; }
  return zwcor;                 // 連続修正の有無を返す
}

bool varclass::cox_man(const option& opt, double& csqrcm, double& csqrcmr, int& dfcmr, double& b, double& seb, const listlf& lifelst0, const vector<mapg_itr>& gitrvec0) const
// Cox-Mantel検定
{
  int s = (int)(lifelst0.size()), s1 = (gitrvec0[0])->second.n();
  double u = 0.0, v = 0.0, cmt = 0.0;
  int nb = 0;
  listlf_citr litr = lifelst0.begin();
  double to = litr->t();
  b = seb = 0.0;
  while (litr != lifelst0.end())
  {
    int dwi = 0, di = 0, d1 = 0, w1 = 0;
    while (litr != lifelst0.end() && litr->t() == to)
    {
      if (litr->cidx() == 0)    // 群1
      {
        if (litr->oc() == ocdeath) ++d1;
        ++w1;
      }
      if (litr->oc() == ocdeath) ++di;
      ++litr;  ++dwi;
    }
    if (s > 1 && di > 0 && s1 < s) // 死亡例があり、2群のデータがある時だけ計算
    {
      double ui = (double)d1 - (double)di * s1 / s;
      double vi = (double)di * (double)(s - di) / (double)s * (double)s1 / (double)s * (double)(s - s1) / (double)(s - 1);
      u += ui;  v += vi;
      if (vi > 0.0) cmt += ui * ui / vi;
      ++nb;
    }
    if (litr != lifelst0.end()) { s -= dwi;  s1 -= w1;  to = litr->t(); }
  }
  dfcmr = nb - 1;               // 異質性の検定の自由度
  if (v <= 0.0) return false;   // 検定できず
  bool cmcor = false;
  if (opt.cmt() == testorg)
  { csqrcm = u * u / v;  cmcor = false; }
  else
  {
    csqrcm = (u - sgn(u) * 0.5) * (u - sgn(u) * 0.5) / v;
    cmcor = true;
  }
  b = -u / v;
  seb = sqrt(1.0 / v);
  csqrcmr = cmt - u * u / v;
  return cmcor;                 // 連続修正の有無を返す
}

bool varclass::caldif_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv, int idx) const
// 傾斜ベクトルgとHessian行列Hを微分法で計算する
{
  constexpr double Emax = 308.0; // 指数最大数
  if (isnan(hv(0)) || isnan(hv(1)) || hv(0) >= Emax || hv(1) >= Emax) return false; // Newton法発散
  size_t nh = hv.dim();         // パラメータ数
  double la = hv(0), a = exp(hv(0)), lh = hv(1), h = exp(hv(1));
  double ha = pow(h, a);
  int nd = 0;
  double sdlt = 0.0, sta = 0.0, stalt = 0.0, stalt2 = 0.0;
  gv = 0.0;  hmat = 0.0;
  for (listlf_citr litr = vlifelst.begin(); litr != vlifelst.end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ計算
    {
      double ti = litr->t();
      if (ti > 0.0)
      {
        double lti = log(ti);
        if (litr->oc() == ocdeath) { ++nd;  sdlt += lti; } // 死亡例
        double tai = pow(ti, a);
        sta += tai;  stalt += tai * lti;  stalt2 += tai * lti * lti;
      }
    }
  }
  ll = (double)nd * (a * lh + la) + (a - 1.0) * sdlt - ha * sta;
  gv.assign(0, (double)(nd) * (a * lh + 1.0) + a * (sdlt - lh * ha * sta - ha * stalt));
  gv.assign(1, a * ((double)nd - ha * sta));
  hmat.assign(0, 0, a * ((double)nd * lh + sdlt - a * lh * lh * ha * sta - 2.0 * a * lh * ha * stalt - a * ha * stalt2 - lh * ha * sta - ha * stalt));
  hmat.assign(0, 1, a * ((double)nd - (a * lh + 1.0) * ha * sta - a * ha * stalt));
  hmat.assign(1, 1, -a * a * ha * sta);
  for (size_t i = 0; i < nh - 1; ++i)
  {
    for (size_t j = i + 1; j < nh; ++j) hmat.assign(j, i, hmat(i, j));
  }
  return true;                  // エラーなし
}

bool varclass::caldel_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv, int idx) const
// 傾斜ベクトルgとHessian行列Hを差分法で計算する
{
  constexpr double Emax = 308.0; // 指数最大数
  if (isnan(hv(0)) || isnan(hv(1)) || hv(0) >= Emax || hv(1) >= Emax) return false; // Newton法発散
  constexpr double Delta = 1.0e-5; // 差分法の相対微小変動
  size_t nh = hv.dim();            // パラメータ数
  ll = wei_llfun(hv, idx);
  if (ll > 0.0 || ll < -DBL_MAX || isnan(ll)) return false; // Newton法発散
  const double Deps = DBL_EPSILON * fmax(5.0, hv.maxabse());
  for (size_t i = 0; i < nh; ++i)
  {
    vec<double> hiv(hv);
    double dhi = (fabs(hv(i)) < Deps) ? Delta : hv(i) * Delta;
    hiv.add(i, dhi);
    double lli = wei_llfun(hiv, idx);
    gv.assign(i, (lli - ll) / dhi);
    for (size_t j = i; j < nh; ++j)
    {
      vec hjv(hv);
      double dhj = (fabs(hv(j)) < Deps) ? Delta : hv(j) * Delta;
      hjv.add(j, dhj);
      double llj = wei_llfun(hjv, idx);
      vec hijv(hiv);
      hijv.add(j, dhj);
      double llij = wei_llfun(hijv, idx);
      hmat.assign(i, j, (llij - lli - llj + ll) / dhi / dhj);
    }
  }
  for (size_t i = 0; i < nh - 1; ++i)
  {
    for (size_t j = i + 1; j < nh; ++j) hmat.assign(j, i, hmat(i, j));
  }
  return true;                  // エラーなし
}

double varclass::wei_llfun(const vec<double>& hv, int idx) const
// Weibull分布モデルの対数尤度を返す
{
  double ll = 0.0, a = exp(hv(0)), lh = hv(1);
  for (listlf_citr litr = vlifelst.begin(); litr != vlifelst.end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ計算
    {
      double ti = litr->t();
      if (ti > 0.0)
      {
        if (litr->oc() == ocdeath) ll += a * lh + hv(0) + (a - 1.0) * log(ti);
        ll += -pow(exp(lh) * ti, a);
      }
    }
  }
  return ll;
}

int varclass::cal(const option& opt, ldata& datalst)
// データを集計して生命表リスト等をセットする
{
  int cno = 1;
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr, ++cno)
  {
    double t = ditr->fdata(1);
    if (ditr->sdata(0) != BLANKs && t != BLANKf)
    {
      octype oc = ocalive;      // 生存
      int nd = 0;
      if (ditr->sdata(2) == opt.dcode()) { oc = ocdeath;  nd = 1; } // 死亡
      else if (ditr->sdata(2) == BLANKs) oc = ocdropout; // 脱落
      life lifei(cno, ditr->sdata(0), t, oc);
      vlifelst.push_back(lifei); // 生命表リストに追加する
      mapg_itr gitr = vgmap.find(ditr->sdata(0));
      if (gitr == vgmap.end())   // 新規群コード
      {
        statlf tstatlf(t, nd);
        vgmap.insert(make_pair(ditr->sdata(0), tstatlf));
      }
      else                      // 既存の群
      { gitr->second.sigma(t, nd); }
    }
  }
  vng = vgmap.size();
  if (vng < 1) return vng;      // 群データなし
  vgitrvec.resize(vng);
  int i = 0;
  for (mapg_itr gitr = vgmap.begin(); gitr != vgmap.end(); ++gitr, ++i) vgitrvec[i] = gitr;
  cal_km(opt);
  return vng;                   // 群数を返す
}

bool varclass::newtonwei(const option& opt, int idx, double lambda)
// Weibull分布モデルをNewton法で計算する
{
  v_hv0.assign(0, log(opt.weia()));  v_hv0.assign(1, log(lambda)); // 初期値
  vec<double> hv(v_hv0), vhv(v_vhv0), gv(v_hv0.dim());
  mat<double> hmat(v_hv0.dim());
  bool cal = true;
  vnr = 0;
  double llpre = 0.0, ll = 0.0; // 対数尤度
  if (opt.wei() == methoddel)   // 差分法
  {
    while (cal && caldel_lgh(gv, hmat, ll, hv, idx))
    {
      ++vnr;
      hmat.inv();
      if (fabs(ll - llpre) < opt.eps() || (opt.rmax() > 0 && vnr >= opt.rmax())) cal = false;
      else { llpre = ll;  hv -= hmat * gv; }
    }
  }
  else                          // 微分法
  {
    while (cal && caldif_lgh(gv, hmat, ll, hv, idx))
    {
      ++vnr;
      hmat.inv();
      if (fabs(ll - llpre) < opt.eps() || (opt.rmax() > 0 && vnr >= opt.rmax())) cal = false;
      else { llpre = ll;  hv -= hmat * gv; }
    }
  }
  if (cal) return false;        // Newton法発散！
  for (size_t i = 0; i < vhv.dim(); ++i) vhv.assign(i, -hmat(i, i));
  v_hv = hv;  v_vhv = vhv;  vll = ll;
  return true;                  // Newton法成功
}

bool varclass::cmtmcom(const option& opt, int iidx, int jidx)
// Cox-Mantelの多重比較を行う
{
  listlf lifelsti;           // 生命表リスト
  for (listlf_citr litr = vlifelst.begin(); litr != vlifelst.end(); ++litr)
  {
    if (litr->cidx() == iidx || litr->cidx() == jidx) // 群1または2
    {
      life lifei(*litr);
      if (litr->cidx() == iidx) lifei.cidx(0);
      else lifei.cidx(1);
      lifelsti.push_back(lifei);
    }
  }
  vector<mapg_itr> gitrvec0(2); // 群別基礎統計量マップ指定子
  gitrvec0[0] = vgitrvec[iidx];  gitrvec0[1] = vgitrvec[jidx];
  vcsqrcm = vcsqrcmr = vb = vseb = 0.0;
  vdfcmr = 0;
  vcmcor = cox_man(opt, vcsqrcm, vcsqrcmr, vdfcmr, vb, vseb, lifelsti, gitrvec0);
  return vcmcor;                // 連続修正の有無を返す
}

class kmclass                   // kmメインクラス
{
private:
  string fname;                 // ファイル名
  void put_resid(ostream& fout, idd& id) const;
  // 解析結果のタイトルとデータ識別情報を出力する
  void put_km(ostream& fout, option& opt, const idd& id, varclass& var) const;
  // Kaplan-Meier法による生命表と各種統計量を出力する
  void put_ltab(ostream& fout, int idx, int ni, const listlf& lifelst) const;
  // 指定された群についてKaplan-Meier法による生命表を出力する
  void put_rmst(ostream& fout, option& opt, double trmst, double rmst, double vrmst) const;
  // RMSTを出力する
  void put_mst(ostream& fout, option& opt, double sp, double sn, double tp, double tn, double tlow, double tup, double tmax) const;
  // 実データによる%生存期間推定値を出力する
  void put_exp(ostream& fout, option& opt, double lambda, double vlambda, double mt, double vmt, double trmst) const;
  // 指数分布モデルによる各種パラメータを出力する
  void put_wei(ostream& fout, option& opt, bool newtonok, const varclass& var) const;
  // Weibull分布モデルのパラメータを出力する
  void put_ltest(ostream& fout, option& opt, const idd& id, varclass& var) const;
  // 各種の検定結果を出力する
  void put_phdt(ostream& fout, option& opt, varclass& var) const;
  // PHD検定の結果を出力する。
  void put_phrt(ostream& fout, option& opt, varclass& var) const;
  // PHR検定の結果を出力する。
  void put_phrtpa(ostream& fout, const option& opt, varclass& var) const;
  // PHR検定の検出力分析結果を出力する。
  void put_cmt(ostream& fout, option& opt, varclass& var) const;
  // Cox-Mantel検定の結果を出力する。
  double put_hci(ostream& fout, const option& opt, char hcode, double h, double seh, int ncom = 1) const;
  // HRの信頼区間を出力する
  void put_gwt(ostream& fout, option& opt, varclass& var) const;
  // 一般化Wilcoxon2標本検定の結果を出力する。
  double put_wci(ostream& fout, const option& opt, double n0, double n1, double w, double vw) const;
  // Wの信頼区間を出力する
  void put_lrt(ostream& fout, option& opt, varclass& var) const;
  // ログランク検定の結果を出力する。
  void put_gemt(ostream& fout, option& opt, varclass& var) const;
  // 一般化拡張Mantel検定の結果を出力する。
  void put_phrtmcom(ostream& fout, const option& opt, const idd& id, const varclass& var) const;
  // PHR検定の多重比較結果を出力する
  void put_cmtmcom(ostream& fout, const option& opt, const idd& id, varclass& var) const;
  // Cox-Mantelの多重比較結果を出力する
  void plot_ngraph(option& opt, idd& id, varclass& var) const;
  // Ngraph用データを出力する
  void plot_ngpid(ofstream& fout, idd& id) const;
  // Ngraph用データのタイトルとデータ識別情報を出力する
  void plot_ngps(ofstream& fout, const option& opt, const idd& id, const varclass& var) const;
  // Ngraph用に累積生存率曲線を出力する
  void plot_ngpsci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const;
  // Ngraph用に累積生存率曲線の信頼区間を出力する
  void plot_ngpsl(ofstream& fout, const option& opt, const idd& id, const varclass& var) const;
  // Ngraph用に累積生存率折れ線を出力する
  void plot_ngpslci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const;
  // Ngraph用に累積生存率折れ線の信頼区間を出力する
  void plot_ngpsew(ofstream& fout, const option& opt, const idd& id, bool wei, varclass& var) const;
  // Ngraph用に指数／Weibull分布モデルによる生存曲線を出力する
  void plot_ngpd(ofstream& fout, const option& opt, const idd& id, const varclass& var) const;
  // Ngraph用に累積死亡率曲線を出力する
  void plot_ngpdci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const;
  // Ngraph用に累積死亡率曲線の信頼区間を出力する
  void plot_ngpdl(ofstream& fout, const option& opt, const idd& id, const varclass& var) const;
  // Ngraph用に累積死亡率折れ線を出力する
  void plot_ngpdlci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const;
  // Ngraph用に累積死亡率折れ線の信頼区間を出力する
  void plot_ngpdew(ofstream& fout, const option& opt, const idd& id, bool wei, varclass& var) const;
  // Ngraph用に指数／Weibull分布モデルによる死亡曲線を出力する
public:
  kmclass() : fname()           // コンストラクタ
  {}
  ~kmclass()                    // デストラクタ
  {}
  int get(const char* c_fname, option& opt, idd& id, ldata& datalst);
  // データを読み込み保存・累積する
  void put(option& opt, idd& id, ldata& datalst) const;
  // 集計結果を出力する
};

void kmclass::put_resid(ostream& fout, idd& id) const
// 解析結果のタイトルとデータ識別情報を出力する
{
  puttitle(fout, Title);
  fout << "\n";
  id.put_title(fout);
  id.put_stra(fout);
  fout << "GROUP ITEM :" << id.item(0) << "\n";
  fout << "PERIOD ITEM:" << id.item(1) << "\n";
  fout << "OUTCOM ITEM:" << id.item(2) << "\n";
}

void kmclass::put_km(ostream& fout, option& opt, const idd& id, varclass& var) const
// Kaplan-Meier法による生命表と各種統計量を出力する
{
  fout << "\n<CUMURATIVE SURVIVAL RATE WITH THE KAPLAN-MEIER METHOD>\n";
  fout << "  death code:" << opt.dcode();
  if (opt.res("ltab")) fout << "  +:censoring  (+):drop out";
  double z = ptoz(1.0 - opt.cc());
  for (int i = 0; i < var.ng(); ++i)
  {
    double st = var.sum(i);
    double tmax = var.max(i);
    int nd = var.nd(i);
    int ni = var.n(i);
    double lambda = (st > 0.0) ? (double)nd / st : 0.0;
    double mt = 0.0, vmt = 0.0, vlambda = 0.0;
    if (nd > 0)
    {
      mt = st / (double)nd;
      vmt = mt * mt / (double)nd;
      vlambda = lambda * lambda / (double)nd;
    }
    double trmst = (opt.rmst() < tmax) ? opt.rmst() : tmax;
    fout << "\nGROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    fout << "  n=" << ni << "  deaths=" << nd << "  total observation period=" << st << "\n";
    if (opt.res("ltab")) put_ltab(fout, i, ni, var.lifelst());
    double sigw = 0.0, sp = 1.0, sn = 1.0;
    double tp = 0.0, tn = 0.0, tlow = 0.0, tup = 0.0, slt = 0.0;
    octype ocpre = ocnon;
    double spre = 1.0, tpre = 0.0, rmst = 0.0, vrmst = 0.0;
    int j = 0;
    for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
    {
      if (litr->cidx() == i)    // 該当する群だけ集計
      {
        double t = litr->t(), s = litr->s();
        if (litr->oc() == ocdeath)
        {
          double w = (ni - j - 1 > 0) ? 1.0 / (double)(ni - j) / (double)(ni - j - 1) : 0.0;
          sigw += w;
          double se = s * sqrt(sigw);
          double zt = (se > 0.0) ? (s - opt.mst()) / se : 0.0;
          if (zt >= z) tlow = t;
          if (tup == 0.0 && -z >= zt) tup = t;
          if (t <= trmst)
          {
            rmst += (t - tpre) * spre;
            double rmsti = 0.0, sprei= s, tprei = t;
            octype ocprei = ocnon;
            listlf_citr litri = litr;
            ++litri;
            while (litri != (var.lifelst()).end())
            {
              double ti = litri->t();
              if (litri->cidx() == i && litri->oc() == ocdeath)
              {
                if (ti <= trmst)
                {
                  rmsti += (ti - tprei) * sprei;
                  sprei = litri->s();  tprei = ti;
                }
              }
              ocprei = litri->oc();
              ++litri;
            }
            if (tmax <= trmst)
            {
              if (ocprei != ocnon && ocprei != ocdeath) rmsti += (tmax - tprei) * sprei;
            }
            vrmst += rmsti * rmsti * w;
            spre = s;  tpre = t;
          }
        }
        if (s >= opt.mst())
        { tp = t;  sp = s; }
        if (tn == 0.0 && opt.mst() >= s)
        { tn = t;  sn = s; }
        ++j;
        slt += exp(-lambda * t);
        ocpre = litr->oc();
      }
    }
    if (tmax <= trmst)
    {
      if (ocpre != ocnon && ocpre != ocdeath) rmst += (tmax - tpre) * spre;
    }
    slt = (double)ni - slt;
    if (opt.res("rmst")) put_rmst(fout, opt, trmst, rmst, vrmst);
    if (opt.res("mst")) put_mst(fout, opt, sp, sn, tp, tn, tlow, tup, tmax);
    if (nd > 0)
    {
      if (opt.res("ex")) put_exp(fout, opt, lambda, vlambda, mt, vmt, trmst);
      if (opt.res("wei"))
      {
        bool newtonok = var.newtonwei(opt, i, lambda);
        put_wei(fout, opt, newtonok, var);
      }
    }
  }
}

void kmclass::put_ltab(ostream& fout, int idx, int ni, const listlf& lifelst) const
// 指定された群についてKaplan-Meier法による生命表を出力する
{
  constexpr int LENline = 75;          // 区切り線の長さ
  ios::fmtflags flags0 = fout.flags(); // 現在の書式を保存する
  fout << "No. Case No. Survival Time(outc)     Sur-n/Obs-n       Survival          SE\n";
  putline(fout, LENline);
  double sigw = 0.0, se = 0.0;
  int j = 0;
  for (listlf_citr litr = lifelst.begin(); litr != lifelst.end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      double t = litr->t();
      fout << setw(4) << j + 1;
      fout << setw(8) << litr->cno();
      fout << setw(14) << t;
      if (litr->oc() == ocdeath)
      {
        double w = (ni - j - 1 > 0) ? 1.0 / (double)(ni - j) / (double)(ni - j - 1) : 0.0;
        sigw += w;
        se = litr->s() * sqrt(sigw);
        fout << setw(16) << ni - j - 1 << "/" << left << setw(8) << ni - j << right;
        fout << setw(12) << litr->s() << setw(12) << se << "\n";
      }
      else
      {
        fout << left;
        if (litr->oc() == ocalive) fout << setw(10) << " +";
        else fout << setw(10) << "(+)";
        fout << right << "(" << setw(5) << ni - j;
        fout << "/" << left << setw(5) << ni - j << right;
        fout << ")" << setw(14) << litr->s() << setw(12) << se << "\n";
      }
      ++j;
    }
  }
  putline(fout, LENline);
  fout.flags(flags0);           // 書式フラッグを元に戻す
}

void kmclass::put_rmst(ostream& fout, option& opt, double trmst, double rmst, double vrmst) const
// RMSTを出力する
{
  fout << "Restricted Mean Survival Time:RMST(t=0-" << trmst << ")=" << rmst << "\n";
  if (opt.res("cc"))
  {
    double se = sqrt(vrmst);
    double z = ptoz(1.0 - opt.cc());
    double rmstl = rmst - z * se, rmstu = rmst + z * se;
    fout << "  " << opt.cc() * 100.0 << "%CI:Lower=" << rmstl << "  Upper=" << rmstu << "\n";
  }
}

void kmclass::put_mst(ostream& fout, option& opt, double sp, double sn, double tp, double tn, double tlow, double tup, double tmax) const
// 実データによる%生存期間推定値を出力する
{
  double psp = opt.mst() * 100.0, ccp = opt.cc() * 100.0;
  fout << psp << "% Survival Time based on Observed Data=";
  if (tn == 0.0)
  { fout << tmax << " or more"; }
  else
  {
    if (sp > sn) tp += (tn - tp) * (sp - opt.mst()) / (sp - sn);
    fout << tp;
  }
  if (opt.res("cc"))
  {
    fout << "\n  " << ccp << "%CI:Lower=" << tlow << "  Upper=";
    if (tup == 0.0) fout << tmax << " or more";
    else fout << tup;
  }
  fout << "\n";
}

void kmclass::put_exp(ostream& fout, option& opt, double lambda, double vlambda, double mt, double vmt, double trmst) const
// 指数分布モデルによる各種パラメータを出力する
{
  fout << "\n<EXPONENTIAL DISTRIBUTION MODEL:MAXIMUM-LIKELIHOOD ESTIMATION>\n";
  fout << "Hazard:lambda=" << lambda;
  double mstp = opt.mst() * 100.0, ccp = opt.cc() * 100.0;
  double z = ptoz(1.0 - opt.cc());
  double se = sqrt(vlambda);
  double ci = z * se;
  double lambdal = lambda - ci, lambdau = lambda + ci;
  if (opt.res("cc")) fout << ccp << "%CI:Lower=" << lambdal << "  Upper=" << lambdau;
  fout << "\n";
  if (opt.res("rmst"))
  {
    double expt = exp(-lambda * trmst), expt2 = exp(-2.0 * lambda * trmst);
    double rmst = (1.0 - expt) / lambda;
    se = sqrt(1.0 - 2.0 * lambda * trmst * expt - expt2) / lambda;
    ci = z * se;
    double rmstl = rmst - ci, rmstu = rmst + ci;
    if (rmstl < 0.0) rmstl = 0.0;
    fout << "RMST(t=0-" << trmst << ")=" << rmst;
    if (opt.res("cc")) fout << ccp << "%CI:Lower=" << rmstl << "  Upper=" << rmstu;
    fout << "\nMean Surv-Time=RMST(t=0-inf)=" << mt;
    se = sqrt(vmt);
    ci = z * se;
    double mtl = mt - ci, mtu = mt + ci;
    if (mtl < 0.0) mtl = 0.0;
    if (opt.res("cc")) fout << ccp << "%CI:Lower=" << mtl << "  Upper=" << mtu;
    fout << "\n";
  }
  if (opt.res("mst"))
  {
    double lps = log(1.0 / opt.mst());
    double t = lps * mt;
    ci = lps * z * se;
    double tl = t - ci, tu = t + ci;
    fout << mstp << "% Suvival Time=" << t;
    if (opt.res("cc")) fout << ccp << "%CI:Lower=" << tl << "  Upper=" << tu;
    fout << "\n";
  }
}

void kmclass::put_wei(ostream& fout, option& opt, bool newtonok, const varclass& var) const
// Weibull分布モデルのパラメータを出力する
{
  fout << "\n<WEIBULL DISTRIBUTION MODEL:";
  if (opt.wei() == methoddel) fout << "DIFFERENCE METHOD";
  else fout << "DIFFERENTIATION METHOD";
  fout << ">\nNumber of Newton Method Repetitions=" << var.nr();
  if (newtonok)
  {
    double a = exp(var.h(0));
    fout << "\nlog(likelihood)=" << var.ll() << "  AIC=" << 2.0 * (2.0 - var.ll()) << "\n";
    fout << "ln(a)=" << var.h(0) << "  a=" << a << "  ln(lambda)=" << var.h(1) << "  lambda=" << exp(var.h(1)) << "\n";
  }
  else
  { fout << "(NEWTON METHOD DIVERGED !)\n"; }
  if (opt.res("v"))             // 計算過程出力
  {
    fout << "hv0(0)=" << var.h0(0) << "  vhv0(0)=" << var.vh0(0) << "  hv0(1)=" << var.h0(1) << "  vhv0(1)=" << var.vh0(1) << "\n";
  }
}

void kmclass::put_ltest(ostream& fout, option& opt, const idd& id, varclass& var) const
// 各種の検定結果を出力する
{
  if (var.dflr() == 1)          // 2群の場合
  {
    if (opt.res("phrt")) put_phrt(fout, opt, var); // PHR検定
    if (opt.res("phdt")) put_phdt(fout, opt, var); // PHD検定
    if (opt.res("cmt")) put_cmt(fout, opt, var); // Cox-Mantel検定
    if (opt.res("gwt")) put_gwt(fout, opt, var); // 一般化Wilcoxonの2標本検定
  }
  if (opt.res("lrt")) put_lrt(fout, opt, var); // ログランク検定
  if (opt.res("gemt") || (var.dfgem() > 1 && opt.res("def"))) put_gemt(fout, opt, var); // 一般化拡張Mantel検定
  if (var.dflr() > 1)
  {
    if (opt.res("mp")) put_phrtmcom(fout, opt, id, var); // PHR検定の多重比較
    if (opt.res("mc")) put_cmtmcom(fout, opt, id, var); // Cox-Mantelの多重比較
  }
  fout << "\n" << endl;
}

void kmclass::put_phdt(ostream& fout, option& opt, varclass& var) const
// PHD検定の結果を出力する。
{
  double st0 = var.sum(0), st1 = var.sum(1);
  int nd0 = var.nd(0), nd1 = var.nd(1);
  double lambda0 = (sgn(st0) > 0) ? (double)nd0 / st0 : 0.0;
  double lambda1 = (sgn(st1) > 0) ? (double)nd1 / st1 : 0.0;
  double hd = lambda1 - lambda0;
  double v0 = (sgn(st0) > 0) ? lambda0 / st0 : 0.0;
  double v1 = (sgn(st1) > 0.0) ? lambda1 / st1 : 0.0;
  double vhd = v0 + v1;
  double sehd = sqrt(vhd);
  double csqr = (vhd > 0.0) ? hd * hd / vhd : 0.0;
  ldbl pr = ctop(csqr, 1);
  fout << "\n<PARAMETRIC HAZARD DIFFERENCE TEST>\n";
  fout << "chi-squared=" << csqr << "  DF=1  p=" << pr << pind(pr) << "\n";
  fout << "HD=λ2 - λ1=" << hd << "  SE of HD=" << sehd << "\n";
  if (opt.res("cc")) put_hci(fout, opt, 'D', hd, sehd, 1);
}

void kmclass::put_phrt(ostream& fout, option& opt, varclass& var) const
// PHR検定の結果を出力する。
{
  double st0 = var.sum(0), st1 = var.sum(1);
  int nd0 = var.nd(0), nd1 = var.nd(1);
  double lambda0 = (sgn(st0) > 0) ? (double)nd0 / st0 : 0.0;
  double lambda1 = (sgn(st1) > 0) ? (double)nd1 / st1 : 0.0;
  double b1 = (sgn(lambda0) > 0) ? log(lambda1 / lambda0) : 0.0;
  double vb1 = (nd0 > 0) ? 1.0 / (double)nd0 : 0.0;
  if (nd1 > 0) vb1 += 1.0 / (double)nd1;
  double seb1 = sqrt(vb1);
  double csqr = (vb1 > 0.0) ? b1 * b1 / vb1 : 0.0;
  ldbl pr = ctop(csqr, 1);
  fout << "\n<PARAMETRIC HAZARD RATIO TEST>\n";
  fout << "chi-squared=" << csqr << "  DF=1  p=" << pr << pind(pr) << "\n";
  fout << "HR=lambda2/lambda1=" << exp(b1) << "  b=ln(HR)=" << b1 << "  SE of b=" << sqrt(vb1) << "\n";
  if (opt.res("cc")) put_hci(fout, opt, 'R', b1, seb1, 1);
  if (opt.res("pa")) put_phrtpa(fout, opt, var);
}

void kmclass::put_phrtpa(ostream& fout, const option& opt, varclass& var) const
// PHR検定の検出力分析結果を出力する。
{
  double st0 = var.sum(0), st1 = var.sum(1);
  double tmax = var.max(0);
  if (var.max(1) > tmax) tmax = var.max(1);
  int nd0 = var.nd(0), nd1 = var.nd(1);
  double lambda0 = (st0 > 0.0) ? (double)nd0 / st0 : 0.0;
  double lambda1 = (st1 > 0.0) ? (double)nd1 / st1 : 0.0;
  double lambda_0 = (opt.lambda1() == BLANKf) ? lambda0 : opt.lambda1();
  double lambda_1 = (opt.lambda2() == BLANKf) ? lambda1 : opt.lambda2();
  if (sgndiff(lambda_0, lambda_1) == 0) lambda_1 = 2.0 * lambda_0; // λ0=λ1の時はλ1=2・λ0にする
  double nd_0 = (opt.lambda1() == BLANKf) ? (double)nd0 : lambda_0 * st0;
  double nd_1 = (opt.lambda2() == BLANKf) ? (double)nd1 : lambda_1 * st1;
  double b1 = (sgn(lambda_0) > 0) ? log(lambda_1 / lambda_0) : 0.0;
  double vb1 = (nd_0 > 0.0) ? 1.0 / nd_0 : 0.0;
  if (nd_1 > 0.0) vb1 += 1.0 / nd_1;
  double seb = sqrt(vb1), za = ptoz(opt.al()), z2b = ptoned(opt.po());
  double zt = fabs(b1) / seb - za;
  double po = nedp(zt);
  fout << "\n<POWER ANALYSIS OF PARAMETRIC HAZARD RATIO TEST>\n";
  fout << "Significance Level=" << opt.al() << "  Observation Period=" << tmax << "\n";
  fout << "HR=" << exp(b1) << "(lambda2=" << lambda_1 << "/lambda1=" << lambda_0 << ")->power=" << po << "\n";
  double b1t = (za + z2b) * seb;
  fout << "power=" << opt.po() << "->HR(lambda2/lambda1)=" << exp(b1t) << "  b=ln(HR)=" << b1t << "\n";
  double exp0 = 1.0 - exp(-lambda_0 * tmax);
  double exp1 = 1.0 - exp(-lambda_1 * tmax);
  zt = (za + z2b) / b1;
  double zt2 = zt * zt;
  double n0 = (exp0 + exp1) / exp0 / exp1 * zt2;
  int n1 = (int)(n0 + 0.99);
  int nd = (int)(n0 * exp0 + n0 * exp1 + 0.99);
  fout << "power=" << opt.po() << "  HR=" << exp(b1) << "->n1=" << n1 << "(total n=" << n1 * 2 << "  deaths=" << nd << ")\n";
}

void kmclass::put_cmt(ostream& fout, option& opt, varclass& var) const
// Cox-Mantel検定の結果を出力する。
{
  ldbl pr = ctop(var.csqrcm(), 1);
  fout << "\n<COX-MANTEL TEST";
  if (var.cmcor()) fout << "(CORRECTION)";
  fout << ">\n";
  fout << "chi-squared=" << var.csqrcm() << "  DF=1  p=" << pr << pind(pr) << "\n";
  pr = ctop(var.csqrcmr(), var.dfcmr());
  fout << "Interaction(Heterogeneity):chi-squared=" << var.csqrcmr() << "  DF=" << var.dfcmr() << "  p=" << pr << pind(pr) << "\n";
  fout << "Cox's b=ln(HR)=" << var.b() << "  SE of b=" << var.seb() << "  HR(GROUP2/GROUP1)=" << exp(var.b()) << "\n";
  if (opt.res("cc")) put_hci(fout, opt, 'R', var.b(), var.seb());
}

double kmclass::put_hci(ostream& fout, const option& opt, char hcode, double h, double seh, int ncom) const
// HD/HRの信頼区間を出力する
{
  double cc = (ncom < 2) ? opt.cc() : 1.0 - (1.0 - opt.cc()) / ncom;
  double ci = ptoz(1.0 - cc) * seh;
  double hl = h - ci, hu = h + ci;
  fout << opt.cc() * 100.0 << "%";
  if (ncom > 1) fout << " Simultaneous";
  fout << " Confidence Interval of ";
  if (hcode == 'D') fout << "HD:";
  else fout << "HR:";
  if (hcode == 'R') fout << "Lower=" << exp(hl) << "  Upper=" << exp(hu) << "\n" << "  b:";
  fout << "Lower=" << hl << "  Upper=" << hu << endl;
  return ci;                    // 信頼幅を返す
}

void kmclass::put_gwt(ostream& fout, option& opt, varclass& var) const
// 一般化Wilcoxn2標本検定の結果を出力する。
{
  ldbl pr = ztop(var.zw());
  fout << "\n<GENERALIZED WILCOXON TEST";
  if (var.zwcor()) fout << "(CORRECTION)";
  fout << ">\nz=" << var.zw() << "  p=" << pr << pind(pr) << "\n";
  int n0 = var.n(0), n1 = var.n(1);
  if (opt.res("cc")) put_wci(fout, opt, (double)n0, (double)n1, var.w(), var.vw());
}

double kmclass::put_wci(ostream& fout, const option& opt, double n0, double n1, double w, double vw) const
// Wの信頼区間を出力する
{
  double nt = n0 + n1;
  double pro = 100.0 / n0 / n1, se = sqrt(vw);
  double ci = ptoz(1.0 - opt.cc()) * se;
  double wl = w - ci, wu = w + ci;
  fout << "GROUP1:W=" << w;
  fout << "  " << opt.cc() * 100.0 << "%CI:Lower=" << wl << "  Upper=" << wu << "\n";
  double u = (w + n0 * (nt + 1.0) - n0 * (n0 + 1.0)) * 0.5;
  double ul = (wl + n0 * (nt + 1.0) - n0 * (n0 + 1.0)) * 0.5;
  double uu = (wu + n0 * (nt + 1.0) - n0 * (n0 + 1.0)) * 0.5;
  fout << "GROUP1:U=" << u << "  Winning Percentage of U=" << u * pro << "%\n";
  fout << opt.cc() * 100.0 << "%CI:Lower=" << ul * pro << "%  Upper=" << uu * pro << "%" << endl;
  return ci;                    // 信頼幅を返す
}

void kmclass::put_lrt(ostream& fout, option& opt, varclass& var) const
// ログランク検定の結果を出力する。
{
  ldbl pr = ctop(var.csqrlr(), var.dflr());
  fout << "\n<LOG-RANK TEST>\n";
  fout << "chi-squared=" << var.csqrlr() << "  DF=" << var.dflr() << "  p=" << pr << pind(pr) << "\n";
}

void kmclass::put_gemt(ostream& fout, option& opt, varclass& var) const
// 一般化拡張Mantel検定の結果を出力する。
{
  ldbl pr = ctop(var.csqrgem(), var.dfgem());
  fout << "\n<GENERALIZED MANTEL-EXTENTION TEST>\n";
  fout << "chi-squared=" << var.csqrgem() << "  DF=" << var.dfgem() << "  p=" << pr << pind(pr) << "\n";
  pr = ctop(var.csqrgemr(), var.dfgemr());
  fout << "Interaction(Heterogeneity):chi-squared=" << var.csqrgemr() << "  DF=" << var.dfgemr() << "  p=" << pr << pind(pr) << "\n";
}

void kmclass::put_phrtmcom(ostream& fout, const option& opt, const idd& id, const varclass& var) const
// PHR検定の多重比較結果を出力する
{
  int ncom = var.ng() * (var.ng() - 1) / 2; // 比較回数
  fout << "\n<BONFERRONI TYPE MULTIPLE COMPARISONS:" << id.item(0) << ">\n";
  for (int i = 0; i < var.ng() - 1; ++i)
  {
    int nd0 = 0, nd1 = 0;
    double st0 = 0.0, st1 = 0.0, lambda0 = 0.0, lambda1 = 0.0;
    for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
    {
      if (litr->cidx() == i)    // 群1
      {
        if (litr->oc() == ocdeath) ++nd0;
        st0 += litr->t();
      }
    }
    lambda0 = (st0 > 0.0) ? (double)nd0 / st0 : 0.0;
    for (int j = i + 1; j < var.ng(); ++j)
    {
      nd1 = 0;  st1 = 0.0;
      for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
      {
        if (litr->cidx() == j)  // 群2
        {
          if (litr->oc() == ocdeath) ++nd1;
          st1 += litr->t();
        }
      }
      lambda1 = (st1 > 0.0) ? (double)nd1 / st1 : 0.0;
      double b0 = 0.0, b1 = 0.0, vb1 = 0.0;
      if (lambda0 >= DBL_EPSILON) { b0 = log(lambda0);  b1 = log(lambda1 / lambda0); }
      if (nd0 > 0) vb1 = 1.0 / (double)nd0;
      if (nd1 > 0) vb1 += 1.0 / (double)nd1;
      double csqr = (vb1 > 0.0) ? b1 * b1 / vb1 : 0.0;
      ldbl pr = (ldbl)ncom * ctop(csqr, 1);
      if (pr > 1.0L) pr = 1.0L;
      fout << var.gcode(i) << " - " << var.gcode(j) << "\n";
      fout << "  Parametric Hazard Ratio test:chi-squared=" << csqr << "  DF=1  p=" << pr << pind(pr) << "\n";
      fout << "  b=ln(HR)=" << b1 << "  SE of b=" << sqrt(vb1) << "  HR(GROUP2/GROUP1)=" << exp(b1) << "\n";
      fout << "  ";  put_hci(fout, opt, 'R', b1, sqrt(vb1), ncom);
    }
  }
}

void kmclass::put_cmtmcom(ostream& fout, const option& opt, const idd& id, varclass& var) const
// Cox-Mantelの多重比較結果を出力する
{
  int ncom = var.ng() * (var.ng() - 1) / 2; // 比較回数
  fout << "\n<BONFERRONI TYPE MULTIPLE COMPARISONS:" << id.item(0) << ">\n";
  for (int i = 0; i < var.ng() - 1; ++i)
  {
    for (int j = i + 1; j < var.ng(); ++j)
    {
      bool cmcor = var.cmtmcom(opt, i, j);
      ldbl pr = (ldbl)ncom * ctop(var.csqrcm(), 1);
      if (pr > 1.0L) pr = 1.0L;
      fout << var.gcode(i) << " - " << var.gcode(j) << "\n";
      fout << "  Cox-Mantel test";
      if (cmcor) fout << "(correction)";
      else fout << "        ";
      fout << ":chi-squared=" << var.csqrcm() << "  DF=1  p=" << pr << pind(pr) << "\n";
      pr = (ldbl)ncom * ctop(var.csqrcmr(), var.dfcmr());
      if (pr > 1.0) pr = 1.0;
      fout << "  Interaction(Heterogeneity):chi-squared=" << var.csqrcmr() << "  DF=" << var.dfcmr() << "  p=" << pr << pind(pr) << "\n";
      fout << "  Cox's b=ln(HR)=" << var.b() << "  SE of b=" << var.seb() << "  HR(GROUP2/GROUP1)=" << exp(var.b()) << "\n";
      fout << "  ";  put_hci(fout, opt, 'R', var.b(), var.seb(), ncom);
    }
  }
}

void kmclass::plot_ngraph(option& opt, idd& id, varclass& var) const
// Ngraph用データを出力する
{
  ofstream fout;
  if (ofopen(fout, (opt.ngpname()).c_str()) == false) return; // ファイルオープン失敗
  bool delml = false;
  if (opt.ngp("s"))             // 累積生存率曲線
  {
    fout << "-s1 -dline -w80 -zy0,1,0.2\n";
    fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
    plot_ngpid(fout, id);
    plot_ngps(fout, opt, id, var);
    delml = true;
  }
  if (opt.ngp("sl"))            // 累積生存率折れ線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, id);
    plot_ngpsl(fout, opt, id, var);
  }
  if (opt.ngp("se"))            // 指数分布モデルによる生存率曲線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, id);
    plot_ngpsew(fout, opt, id, false, var);
  }
  if (opt.ngp("sw"))            // Weibull分布モデルによる生存率曲線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, id);
    plot_ngpsew(fout, opt, id, true, var);
  }
  if (opt.ngp("d"))             // 累積死亡率曲線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, id);
    plot_ngpd(fout, opt, id, var);
  }
  if (opt.ngp("dl"))            // 累積死亡率折れ線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, id);
    plot_ngpdl(fout, opt, id, var);
  }
  if (opt.ngp("de"))            // 指数分布モデルによる死亡率曲線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
      delml = true;
    }
    plot_ngpid(fout, id);
    plot_ngpdew(fout, opt, id, false, var);
  }
  if (opt.ngp("dw"))            // Weibull分布モデルによる死亡率曲線
  {
    if (delml)
    { fout << NGPdelm << "\n"; }
    else
    {
      fout << "-s1 -dline -w80 -zy0,1,0.2\n";
      fout << "%When Drawing Drop Out Cases:-derrorbar_y -w40 -o0 -zy0,1,0.2\n";
    }
    plot_ngpid(fout, id);
    plot_ngpdew(fout, opt, id, true, var);
  }
}

void kmclass::plot_ngpid(ofstream& fout, idd& id) const
// Ngraph用データのタイトルとデータ識別情報を出力する
{
  fout << "%";  puttitle(fout, Title);  fout << "%\n";
  id.put_title(fout, formngp);
  id.put_stra(fout, formngp);
  fout << "%GROUP ITEM :" << id.item(0) << "\n";
  fout << "%PERIOD ITEM:" << id.item(1) << "\n";
  fout << "%OUTCOM ITEM:" << id.item(2) << "\n";
}

void kmclass::plot_ngps(ofstream& fout, const option& opt, const idd& id, const varclass& var) const
// Ngraph用に累積生存率曲線を出力する
{
  for (int i = 0; i < var.ng(); ++i)
  {
    if (i > 0) fout << NGPdelm << "\n";
    fout << "%<CUMULATIVE SURVIVAL CURVE>\n";
    fout << "%GROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    fout << "0 1\n";
    double spre = 1.0;
    for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
    {
      if (litr->cidx() == i)    // 該当する群だけ出力
      {
        fout << litr->t() << " " << spre;
        if (litr->oc() == ocdeath) fout << "\n" << litr->t() << " " << litr->s() << "\n";
        else fout << " 0 0.02\n";
        spre = litr->s();
      }
    }
    if (0.0 < opt.cc() && opt.cc() < 1.0) plot_ngpsci(fout, opt, id, i, var);
  }
}

void kmclass::plot_ngpsci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const
// Ngraph用に累積生存率曲線の信頼区間を出力する
{
  double z = ptoz(1.0 - opt.cc());
  fout << NGPdelm << "\n";
  fout << "%<" << 100.0 * opt.cc() << "%CI OF CUMULATIVE SURVIVAL CURVE>\n";
  fout << "%GROUP" << idx + 1 << ":" << id.item(0) << "=" << var.gcode(idx) << "\n";
  fout << "%Upper Limit\n";
  int ni = var.n(idx);
  int j = 0;
  double sci = 1.0, ci = 0.0, spre = 1.0, sigw = 0.0;
  fout << "0 1\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      fout << litr->t() << " " << spre;
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
        sci = litr->s() + ci;
        if (sci > 1.0) sci = 1.0;
        fout << "\n" << litr->t() << " " << sci;
      }
      fout << "\n";
      spre = sci;  ++j;
    }
  }
  fout << "= =\n";
  fout << "%Lower Limit\n";
  j = 0;  sci = 1.0;  spre = 1.0;  sigw = 0.0;
  fout << "0 1\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      fout << litr->t() << " " << spre;
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
        sci = litr->s() - ci;
        if (sci < 0.0) sci = 0.0;
        fout << "\n" << litr->t() << " " << sci;
      }
      fout << "\n";
      spre = sci;  ++j;
    }
  }
}

void kmclass::plot_ngpsl(ofstream& fout, const option& opt, const idd& id, const varclass& var) const
// Ngraph用に累積生存率折れ線を出力する
{
  for (int i = 0; i < var.ng(); ++i)
  {
    if (i > 0) fout << NGPdelm << "\n";
    fout << "%<CUMULATIVE SURVIVAL LINE>\n";
    fout << "%GROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    fout << "0 1\n";
    double spre = 1.0;
    for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
    {
      if (litr->cidx() == i)    // 該当する群だけ出力
      {
        fout << litr->t() << " ";
        if (litr->oc() == ocdeath) fout << litr->s() << "\n";
        else fout << litr->s() << " 0 0.02\n";
        spre = litr->s();
      }
    }
    plot_ngpslci(fout, opt, id, i, var);
  }
}

void kmclass::plot_ngpslci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const
// Ngraph用に累積生存率折れ線の信頼区間を出力する
{
  double z = ptoz(1.0 - opt.cc());
  fout << NGPdelm << "\n";
  fout << "%<" << 100.0 * opt.cc() << "%CI OF CUMULATIVE SURVIVAL LINE>\n";
  fout << "%GROUP" << idx + 1 << ":" << id.item(0) << "=" << var.gcode(idx) << "\n";
  fout << "%Upper Limit\n";
  int ni = var.n(idx);
  int j = 0;
  double sci = 1.0, ci = 0.0, spre = 1.0, sigw = 0.0;
  fout << "0 1\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
      }
      sci = litr->s() + ci;
      if (sci > 1.0) sci = 1.0;
      fout << litr->t() << " " << sci << "\n";
      spre = sci;  ++j;
    }
  }
  fout << "= =\n";
  fout << "%Lower Limit\n";
  j = 0;  sci = 1.0;  spre = 1.0;  sigw = 0.0;
  fout << "0 1\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
      }
      sci = litr->s() - ci;
      if (sci < 0.0) sci = 0.0;
      fout << litr->t() << " " << sci << "\n";
      spre = sci;  ++j;
    }
  }
}

void kmclass::plot_ngpsew(ofstream& fout, const option& opt, const idd& id, bool wei, varclass& var) const
// Ngraph用に指数／Weibull分布モデルによる生存曲線を出力する
{
  listlf_citr litr = (var.lifelst()).end();
  --litr;
  double tmax = litr->t() * 1.1; // 観測期間最大値x1.1
  double ltmax = log(tmax);
  double lts = log(tmax / 100.0);
  double ldt = (ltmax - lts) / 20.0;
  double a = 1.0;
  for (int i = 0; i < var.ng(); ++i)
  {
    double st = var.sum(i);
    int nd = var.nd(i);
    double lambda = (st > 0.0) ? (double)nd / st : 0.0;
    if (i > 0) fout << NGPdelm << "\n";
    if (wei)                    // Weibull分布
    {
      fout << "%<SURVIVAL CURVE WITH WEIBULL DISTRIBUTION MODEL>\n";
      fout << "%Number of Newton Method Repetitions";
      if (opt.wei() == methoddel) fout << "%(Difference Method)=";
      else fout << "%(Differentiation Method)=";
      if (var.newtonwei(opt, i, lambda))
      {
        a = exp(var.h(0));  lambda = exp(var.h(1));
        fout << var.nr() << "  ln(a)=" << var.h(0) << "  a=" << a << "  ln(lambda)=" << var.h(1) << "  lambda=" << lambda << "\n";
      }
      else
      {
        fout << var.nr() << "(NEWTON METHOD DIVERGED !)\n";
        return;
      }
    }
    else
    { fout << "%<SURVIVAL CURVE WITH EXPONENTIAL DISTRIBUTION MODEL>\n"; }
    fout << "%GROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    fout << "0 1\n";
    for (double lt = lts; lt < ltmax; lt += ldt)
    {
      double t = exp(lt);
      fout << t << " " << exp(-pow(lambda * t, a)) << "\n";
    }
    fout << tmax << " " << exp(-pow(lambda * tmax, a)) << "\n";
  }
}

void kmclass::plot_ngpd(ofstream& fout, const option& opt, const idd& id, const varclass& var) const
// Ngraph用に累積死亡率曲線を出力する
{
  for (int i = 0; i < var.ng(); ++i)
  {
    if (i > 0) fout << NGPdelm << "\n";
    fout << "%<CUMULATIVE MORTALITY CURVE>\n";
    fout << "%GROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    double spre = 1.0;
    fout << "0 0\n";
    for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
    {
      if (litr->cidx() == i)    // 該当する群だけ出力
      {
        fout << litr->t() << " " << 1.0 - spre;
        if (litr->oc() == ocdeath) fout << "\n" << litr->t() << " " << 1.0 - litr->s() << "\n";
        else fout << " 0 0.02\n";
        spre = litr->s();
      }
    }
    if (0.0 < opt.cc() && opt.cc() < 1.0) plot_ngpdci(fout, opt, id, i, var);
  }
}

void kmclass::plot_ngpdci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const
// Ngraph用に累積死亡率曲線の信頼区間を出力する
{
  double z = ptoz(1.0 - opt.cc());
  fout << NGPdelm << "\n";
  fout << "%<" << 100.0 * opt.cc() << "%CI OF CUMULATIVE MORTALITY CURVE>\n";
  fout << "%GROUP" << idx + 1 << ":" << id.item(0) << "=" << var.gcode(idx) << "\n";
  fout << "%Lower Limit\n";
  int ni = var.n(idx);
  int j = 0;
  double sci = 1.0, ci = 0.0, spre = 1.0, sigw = 0.0;
  fout << "0 0\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      fout << litr->t() << " " << 1.0 - spre;
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
        sci = litr->s() + ci;
        if (sci > 1.0) sci = 1.0;
        fout << "\n" << litr->t() << " " << 1.0 - sci;
      }
      fout << "\n";
      spre = sci;  ++j;
    }
  }
  fout << "= =\n";
  fout << "%Upper Limit\n";
  j = 0;  sci = 1.0;  spre = 1.0;  sigw = 0.0;
  fout << "0 0\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      fout << litr->t() << " " << 1.0 - spre;
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
        sci = litr->s() - ci;
        if (sci < 0.0) sci = 0.0;
        fout << "\n" << litr->t() << " " << 1.0 - sci;
      }
      fout << "\n";
      spre = sci;  ++j;
    }
  }
}

void kmclass::plot_ngpdl(ofstream& fout, const option& opt, const idd& id, const varclass& var) const
// Ngraph用に累積死亡率折れ線を出力する
{
  for (int i = 0; i < var.ng(); ++i)
  {
    if (i > 0) fout << NGPdelm << "\n";
    fout << "%<CUMULATIVE MORTALITY LINE>\n";
    fout << "%GROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    fout << "0 0\n";
    double spre = 1.0;
    for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
    {
      if (litr->cidx() == i)    // 該当する群だけ出力
      {
        fout << litr->t() << " ";
        if (litr->oc() == ocdeath) fout << 1.0 - litr->s() << "\n";
        else fout << 1.0 - litr->s() << " 0 0.02\n";
        spre = litr->s();
      }
    }
    plot_ngpdlci(fout, opt, id, i, var);
  }
}

void kmclass::plot_ngpdlci(ofstream& fout, const option& opt, const idd& id, int idx, const varclass& var) const
// Ngraph用に累積死亡率折れ線の信頼区間を出力する
{
  double z = ptoz(1.0 - opt.cc());
  fout << NGPdelm << "\n";
  fout << "%<" << 100.0 * opt.cc() << "%CI OF CUMULATIVE MORTALITY LINE>\n";
  fout << "%GROUP" << idx + 1 << ":" << id.item(0) << "=" << var.gcode(idx) << "\n";
  fout << "%Lower Limit\n";
  int ni = var.n(idx);
  int j = 0;
  double sci = 1.0, ci = 0.0, spre = 1.0, sigw = 0.0;
  fout << "0 0\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
      }
      sci = litr->s() + ci;
      if (sci > 1.0) sci = 1.0;
      fout << litr->t() << " " << 1.0 - sci << "\n";
      spre = sci;  ++j;
    }
  }
  fout << "= =\n";
  fout << "%Upper Limit\n";
  j = 0;  sci = 1.0;  spre = 1.0;  sigw = 0.0;
  fout << "0 0\n";
  for (listlf_citr litr = (var.lifelst()).begin(); litr != (var.lifelst()).end(); ++litr)
  {
    if (litr->cidx() == idx)    // 該当する群だけ出力
    {
      if (litr->oc() == ocdeath)
      {
        if (j + 1 < ni) sigw += 1.0 / (double)(ni - j) / (double)(ni - j - 1);
        ci = z * litr->s() * sqrt(sigw);
      }
      sci = litr->s() - ci;
      if (sci < 0.0) sci = 0.0;
      fout << litr->t() << " " << 1.0 - sci << "\n";
      spre = sci;  ++j;
    }
  }
}

void kmclass::plot_ngpdew(ofstream& fout, const option& opt, const idd& id, bool wei, varclass& var) const
// Ngraph用に指数／Weibull分布モデルによる死亡曲線を出力する
{
  listlf_citr litr = (var.lifelst()).end();
  --litr;
  double tmax = litr->t() * 1.1; // 観測期間最大値x1.1
  double ltmax = log(tmax);
  double lts = log(tmax / 100.0);
  double ldt = (ltmax - lts) / 20.0;
  double a = 1.0;
  for (int i = 0; i < var.ng(); ++i)
  {
    double st = var.sum(i);
    int nd = var.nd(i);
    double lambda = (st > 0.0) ? (double)nd / st : 0.0;
    if (i > 0) fout << NGPdelm << "\n";
    if (wei)                    // Weibull分布
    {
      fout << "%<MORTALITY CURVE WITH WEIBULL DISTRIBUTION MODEL>\n";
      fout << "%Number of Newton Method Repetitions";
      if (opt.wei() == methoddel) fout << "%(Difference Method)=";
      else fout << "%(Differentiation Method)=";
      if (var.newtonwei(opt, i, lambda))
      {
        a = exp(var.h(0));  lambda = exp(var.h(1));
        fout << var.nr() << "  ln(a)=" << var.h(0) << "  a=" << a << "  ln(lambda)=" << var.h(1) << "  lambda=" << lambda << "\n";
      }
      else
      {
        fout << var.nr() << "(NEWTON METHOD DIVERGED !)\n";
        return;
      }
    }
    else
    { fout << "%<MORTALITY CURVE WITH EXPONENTIAL DISTRIBUTION MODEL>\n"; }
    fout << "%GROUP" << i + 1 << ":" << id.item(0) << "=" << var.gcode(i) << "\n";
    fout << "0 0\n";
    for (double lt = lts; lt < ltmax; lt += ldt)
    {
      double t = exp(lt);
      fout << t << " " << 1.0 - exp(-pow(lambda * t, a)) << "\n";
    }
    fout << tmax << " " << 1.0 - exp(-pow(lambda * tmax, a)) << "\n";
  }
}

int kmclass::get(const char* c_fname, option& opt, idd& id, ldata& datalst)
// データを読み込み保存・累積する
{
  if (c_fname == nullptr) fname.erase();
  else fname = c_fname;
  int nc = fgetdata(fname, id, datalst);
  if (nc < 0) throw runtime_error("Run-time error !");
  if (opt.check(id.ni()) != 0) throw option_error("Run-time option error !");
  return nc;
}

void kmclass::put(option& opt, idd& id, ldata& datalst) const
// 集計結果を出力する
{
  if (id.nc() < 1)              // データなし
  {
    if (opt.res("res"))
    {
      put_resid(cout, id);
      cout << "\n" << endl;
    }
    return;
  }
  varclass var(NPwei);                   // 変数オブジェクト
  if (var.cal(opt, datalst) < 1) return; // 有効なデータなし
  if (opt.res("res"))
  {
    put_resid(cout, id);
    put_km(cout, opt, id, var);
    put_ltest(cout, opt, id, var);
  }
  if (opt.ngpname() != "") plot_ngraph(opt, id, var);
}

int main(int argc, char* argv[]) // メインルーチン
{
  kmclass km;                   // メインクラス変数
  option opt(argv[0]);          // オプション
  idd id;                       // データ識別情報
  ldata datalst;                // データリスト
  try
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理フラッグ
    for (int i = 1; i <= argc; ++i)
    {
      if (i >= argc)
      {
        if (fok) km.get(nullptr, opt, id, datalst); // 標準入力から読み込む
      }
      else if (*(argv[i]) == CHRopt)
      { opt.set(argv[i] + 1);  fok = true; }
      else
      { km.get(argv[i], opt, id, datalst);  fok = false; }
    }
    km.put(opt, id, datalst);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "Unknown error occurred !");
    return 255;
  }
  return 0;
}
