// phte.C Version 7.41e  Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-09 14:29:08 tac>
//
// パラメトリック・ハザード検定を行う。
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法  :g++ -O -s -W phte phte.C
//               -I/usr/local/include -L/usr/local/lib -ldans -lfun

#include <danse.H>              // dans用ヘッダファイル(fune.H,mate.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法表示用
{
  "phte  Version 7.41e          Copyleft(Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:phte [<OPTION> ...] [<DATA FILE> ...]",
  "  The parametric hazard test with the first item as the group, the second",
  "  item as the number of deaths, the third item as total observation period.",
  "<OPTION>",
  "  -cc[<C-COEFFICIENT>]:confidence interval(0=<C-COEFFICIENT<1)",
  "  -f=<FILE>:reads options from the specified file",
  "  -h[=<MAN COMMAND>]:show online manual(-h:-h=man)",
  "  -m[={-|+}]:specify whether to output multiple comparisons",
  "    -:no output  +:output(default)  -m:-m=+",
  "  -phdt[={-|+}]:specify whether to output the parametric hazard difference test",
  "    -:no output  +:output(default)  -phdt:-phdt=+",
  "  -phrt[={-|+}]:specify whether to output the parametric hazard ratio test",
  "    -:no output  +:output(default)  -phrt:-phrt=+"
};

const string Title = "=== Parametric Hazard Test ==="; // 表題

class option                    // オプションクラス
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  double vcc;                   // 信頼係数
  bool vm;                      // 多重比較の出力フラッグ
  bool vphdt;                   // パラメトリック・ハザード差検定の出力フラッグ
  bool vphrt;                   // パラメトリック・ハザード比検定の出力フラッグ
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  double cc() const             // 信頼係数を返す
  { return vcc; }
  bool m() const                // 多重比較の出力フラッグを返す
  { return vm; }
  bool phdt() const // パラメトリック・ハザード差検定の出力フラッグを返す
  { return vphdt; }
  bool phrt() const // パラメトリック・ハザード比検定の出力フラッグを返す
  { return vphrt; }
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check(size_t ni);         // オプションのチェック
};

option::option(const char* c_pname) : vpath(), vpname(), vcc(CCdef), vm(true), vphdt(true), vphrt(true)
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "cc")           // 信頼係数の指定
  {
    vcc = opt[1].empty() ? CCdef : strtodf(opt[1]);
    if (vcc < 0.0 || 1.0 <= vcc) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "m")       // 多重比較結果の出力指定
  {
    if (opt[1].empty() || opt[1] == "+") vm = true;
    else if (opt[1] == "-") vm = false;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "phdt")    // パラメトリック・ハザード差検定の出力指定
  {
    if (opt[1].empty() || opt[1] == "+") vphdt = true;
    else if (opt[1] == "-") vphdt = false;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "phrt")    // パラメトリック・ハザード比検定の出力指定
  {
    if (opt[1].empty() || opt[1] == "+") vphrt = true;
    else if (opt[1] == "-") vphrt = false;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this help message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

int option::check(size_t ni)    // オプションのセットとチェック
{
  if (ni != 3) return disperr(vpname, "Number of items must be 3 !");
  return 0;                     // エラーなし
}

class varclass                  // 変数クラス
{
private:
  int vng;                      // 群数
  vector<string> gvec;          // 群コードベクトル
  vecdbl dvec;                  // 死亡例数ベクトル
  vecdbl tvec;                  // 観察期間合計ベクトル
  vecdbl lvec;                  // ハザードベクトル
public:
  varclass(size_t nc0) : vng(0), gvec(nc0, BLANKs), dvec(nc0, 0.0), tvec(nc0, 0.0), lvec(nc0, 0.0)
  // コンストラクタ
  {}
  ~varclass()                   // デストラクタ
  {}
  int cal(ldata& datalst);      // データを集計して変数をセットする
  int ng() const                // 群数を返す
  { return vng; }
  const string& g(size_t idx) const // 群コードを返す
  { return (idx < gvec.size()) ? gvec[idx] : BLANKs; }
  double d(size_t idx) const    // 死亡例数を返す
  { return (idx < dvec.size()) ? dvec[idx] : 0.0; }
  double t(size_t idx) const    // 観察期間合計を返す
  { return (idx < tvec.size()) ? tvec[idx] : 0.0; }
  double l(size_t idx) const    // ハザードを返す
  { return (idx < lvec.size()) ? lvec[idx] : 0.0; }
};

int varclass::cal(ldata& datalst) // データを集計して変数をセットする
{
  vng = 0;
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr)
  {
    gvec[vng] = ditr->sdata(0);
    dvec[vng] = ditr->fdata(1);  tvec[vng] = ditr->fdata(2);
    if (dvec[vng] != BLANKf && dvec[vng] >= 0.0 && tvec[vng] != BLANKf && tvec[vng] > 0.0)
    {
      lvec[vng] = dvec[vng] / tvec[vng];
      ++vng;
    }
  }
  return vng;                   // 群数を返す
}

class phtclass                  // phtメインクラス
{
private:
  string fname;                 // ファイル名
  void put_id(ostream& fout, idd& id) const;
  // タイトルとデータ識別情報を出力する
  void put_hazard(ostream& fout, const option& opt, const varclass& var) const;
  // ハザードを出力する
  void put_phrt(ostream& fout, const option& opt, const varclass& var) const;
  // パラメトリックハザード比検定の結果と推定結果を出力する
  void put_phdt(ostream& fout, const option& opt, const varclass& var) const;
  // パラメトリックハザード差検定の結果と推定結果を出力する
public:
  phtclass() : fname()          // コンストラクタ
  {}
  ~phtclass()                   // デストラクタ
  {}
  int get(const char* c_fname, option& opt, idd& id, ldata& datalst);
  // データを読み込む
  void put(option& opt, idd& id, ldata& datalst) const;
  // 検定結果を出力する
};

void phtclass::put_id(ostream& fout, idd& id) const
// タイトルとデータ識別情報を出力する
{
  puttitle(fout, Title);
  fout << "\n";
  id.put_title(fout);
  id.put_stra(fout);
  fout << "GROUP ITEM :" << id.item(0) << "\n";
  fout << "DEATHS ITEM:" << id.item(1) << "\n";
  fout << "TOTAL PERIOD ITEM:" << id.item(2) << "\n";
}

void phtclass::put_hazard(ostream& fout, const option& opt, const varclass& var) const
// ハザードを出力する
{
  constexpr int LENline = 70;   // 区切り線の長さ
  fout << "\nGROUP    n of Deaths  Total Period      Hazard";
  fout << right << setw(3) << opt.cc() * 100.0 << "%CI:Lower       Upper\n";
  double z = ptoz(1.0 - opt.cc());
  putline(fout, LENline);
  for (int i = 0; i < var.ng(); ++i)
  {
    string gcode(var.g(i));
    int npad = gcode.size() - strclen(gcode, CHARwide);
    fout << left << setw(10 + npad) << gcode;
    fout << right << setw(10) << var.d(i) << setw(14) << var.t(i);
    fout << setw(12) << var.l(i);
    double vl = (var.t(i) > 0.0) ? var.l(i) / var.t(i) : 0.0;
    if (0.0 < opt.cc() && opt.cc() < 1.0)
    {
      double ci = z * sqrt(vl);
      double ll = var.l(i) - ci, lu = var.l(i) + ci;
      fout << setw(12) << ll << setw(12) << lu << "\n";
    }
    else
    { fout << setw(12) << sqrt(vl) << "\n"; }
  }
  putline(fout, LENline);
}


void phtclass::put_phrt(ostream& fout, const option& opt, const varclass& var) const
// パラメトリックハザード比検定の結果と推定結果を出力する
{
  int ncom = (opt.m()) ? var.ng() * (var.ng() - 1) / 2 : 1; // 比較回数
  for (int i = 0; i < var.ng() - 1; ++i)
  {
    double v1 = (var.d(i) > 0.0) ? 1.0 / var.d(i) : 0.0;
    for (int j = i + 1; j < var.ng(); ++j)
    {
      double v2 = (var.d(j) > 0.0) ? 1.0 / var.d(j) : 0.0;
      double b = (var.l(j) > 0.0 && var.l(i) > 0.0) ? log(var.l(j) / var.l(i)) : 0.0;
      double vb = v1 + v2;
      double csqr = (vb > 0.0) ? b * b / vb : 0.0;
      fout << var.g(j) << "/" << var.g(i) << ":";
      fout << "HR(Hazard Ratio)=" << exp(b) << "  ln(HR)=" << b << "\n";
      double cc = (ncom > 1) ? 1.0 - (1.0 - opt.cc()) / ncom : opt.cc();
      double ci = ptoz(1.0 - cc) * sqrt(vb);
      double bl = b - ci, bu = b + ci;
      fout << opt.cc() * 100.0 << "%";
      if (ncom > 1) fout << " simu-";
      fout << "CI:HRl=" << exp(bl) << "  HRu=" << exp(bu);
      fout << "  ln(HRl)=" << bl << "  ln(HRu)=" << bu << "\n";
      ldbl pr = (ldbl)ncom * ctop(csqr, 1);
      if (pr > 1.0L) pr = 1.0L;
      fout << "Parametric Hazrd Ratio Test";
      if (ncom > 1) fout << "(Bonferroni type Multiple Comparison)";
      fout << ":\n";
      fout << "  chi-squared=" << csqr << "  DF=1  p=" << pr << pind(pr) << "\n";
    }
  }
}

void phtclass::put_phdt(ostream& fout, const option& opt, const varclass& var) const
// パラメトリックハザード差検定の結果と推定結果を出力する
{
  int ncom = (opt.m()) ? var.ng() * (var.ng() - 1) / 2 : 1; // 比較回数
  for (int i = 0; i < var.ng() - 1; ++i)
  {
    double v1 = (var.t(i) > 0.0) ? var.l(i) / var.t(i) : 0.0;
    for (int j = i + 1; j < var.ng(); ++j)
    {
      double v2 = (var.t(j) > 0.0) ? var.l(j) / var.t(j) : 0.0;
      double ld = var.l(j) - var.l(i);
      double vd = v1 + v2;
      double csqr = (vd > 0.0) ? ld * ld / vd : 0.0;
      fout << var.g(j) << " - " << var.g(i) << ":";
      fout << "HD(Hazard Difference)=" << ld << "\n";
      double cc = (ncom > 1) ? 1.0 - (1.0 - opt.cc()) / ncom : opt.cc();
      double ci = ptoz(1.0 - cc) * sqrt(vd);
      double ldl = ld - ci, ldu = ld + ci;
      fout << opt.cc() * 100.0 << "%";
      if (ncom > 1) fout << " Simulaneous ";
      fout << "CI:HDl=" << ldl << "  HDu=" << ldu << "\n";
      ldbl pr = (ldbl)ncom * ctop(csqr, 1);
      if (pr > 1.0L) pr = 1.0L;
      fout << "Parametric Hazard Difference Test";
      if (ncom > 1) fout << "(Bonferroni Type Multiple Comparison)";
      fout << ":\n";
      fout << "  chi-squared=" << csqr << "  DF=1  p=" << pr << pind(pr) << "\n";
    }
  }
}

int phtclass::get(const char* c_fname, option& opt, idd& id, ldata& datalst)
// データを読み込む
{
  if (c_fname == nullptr) fname.erase();
  else fname = c_fname;
  int nc = fgetdata(fname, id, datalst);
  if (nc < 0) throw runtime_error("Run-time error !");
  if (opt.check(id.ni()) != 0) throw option_error("Run-time option error !");
  return nc;
}

void phtclass::put(option& opt, idd& id, ldata& datalst) const
// 検定結果を出力する
{
  put_id(cout, id);
  if (id.nc() < 1)              // データなし
  {
    cout << "\n" << endl;
    return;
  }
  varclass var(id.nc());                 // 変数オブジェクト
  if (var.cal(datalst) < 1) return;      // 有効なデータなし
  put_hazard(cout, opt, var);
  if (opt.phrt()) put_phrt(cout, opt, var);
  if (opt.phdt())
  {
    if (opt.phrt()) cout << "\n";
    put_phdt(cout, opt, var);
  }
  cout << "\n" << endl;
}

int main(int argc, char* argv[]) // メインルーチン
{
  phtclass pht;                 // メインクラス変数
  option opt(argv[0]);          // オプション
  idd id;                       // データ識別情報
  ldata datalst;                // データリスト
  try
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理フラッグ
    for (int i = 1; i <= argc; ++i)
    {
      if (i >= argc)
      {
        if (fok) pht.get(nullptr, opt, id, datalst); // 標準入力から読み込む
      }
      else if (*(argv[i]) == CHRopt)
      { opt.set(argv[i] + 1);  fok = true; }
      else
      { pht.get(argv[i], opt, id, datalst);  fok = false; }
    }
    pht.put(opt, id, datalst);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "Unknown error occurred !");
    return 255;
  }
  return 0;
}
