// pltcve.C Version 7.41e  Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-09 15:03:12 tac>
//
// パラメトリック多変量生命表解析の交差検証
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法  :g++ -O -s -W pltcve pltcve.C
//               -I/usr/localE/include -L/usr/local/lib -ldanse -lfune

#include <danse.H>              // dans用ヘッダファイル(fune.H,mate.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法表示用
{
  "pltcve  Version 7.41e       Copyleft(Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:pltcve [<OPTION> ...] [<DATA FILE> ...]",
  "  The cross validation of parametric survival time analysis with the first",
  "  item as the period, the second item as the outcome, and the subsequent",
  "  items as the covariates.",
  "<OPTION>",
  "  -b=<INITIAL VALUE OF BETA-0>[,<INITIAL VALUE OF BETA-1>,...]",
  "    :specify initial values for partial regression coefficients",
  "  -cc[=<C-COEFFICIENT>]:confidence interval(0=<C-COEFFICIENT<1)",
  "  -d=<CODE>:specify the code of death(default:-d=0)",
  "  -eps[=<ERROR>]:specify the convergence error of the maximum likelihood",
  "    method(0<ERROR)",
  "  -f=<FILE>:reads options from the specified file",
  "  -h[=<MAN COMMAND>]:show online manual(-h:-h=man)",
  "  -method[={del|dif}]:select a calculation method(-method:-method=dif)",
  "  -model[={exp|wei}]:select a distribution model(-model:-model=exe)",
  "  -rmax=<NUMBER>:specify the maximum number of iterations(0=<NUMBER)",
  "  -t=<PERIOD-1>[,<PERIDO-2>,…]:specify the verification periods"
};

constexpr double EPSdef = 1.0e-5; // 省略時収束誤差
constexpr int RMAXdef = 200;      // 省略時最大繰り返し数

using listvecd = list<vecdbl>;    // ベクトルデータの短縮名定義
using listvecd_citr = list<vecdbl>::const_iterator; // 反復子の短縮名定義

enum methodtype                 // 計算方法の種類
{
  methodnon,                    // 計算方法指定なし
  methoddel,                    // 差分法
  methoddif,                    // 微分法
  methodetc                     // その他
};

enum modeltype                  // 分布モデルの種類
{
  modelnon,                     // モデル指定なし
  modelexp,                     // 指数分布モデル
  modelwei,                     // Weibull分布モデル
  modeletc                      // その他
};

class option                    // オプション集合体
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  lstr blst;                    // 偏回帰係数の初期値リスト
  vecdbl vbvec;                 // 偏回帰係数の初期値ベクトル
  double vcc;                   // 信頼係数
  string vdcode;                // 死亡のコード
  double veps;                  // 最尤法の収束誤差
  methodtype vmethod;           // 計算方法の種類
  modeltype vmodel;             // 分布モデルの種類
  int vrmax;                    // 最大計算回数
  lstr tlst;                    // 生存時間リスト
  vecdbl vtvec;                 // 生存時間ベクトル
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  double b(size_t idx) const    // 偏回帰係数初期値を返す
  { return (idx < vbvec.size()) ? vbvec[idx] : BLANKf; }
  double cc() const             // 信頼係数を返す
  { return vcc; }
  const string& dcode() const   // 死亡のコードを返す
  { return vdcode; }
  double eps() const            // 最尤法の収束誤差を返す
  { return veps; }
  methodtype method() const     // 計算方法の種類を返す
  { return vmethod; }
  modeltype model() const       // 分布モデルの種類を返す
  { return vmodel; }
  int rmax() const              // 最大計算回数を返す
  { return vrmax; }
  size_t nt() const             // 生存時間数を返す
  { return vtvec.size(); }
  double t(size_t tidx) const   // 生存時間を返す
  { return (tidx < vtvec.size()) ? vtvec[tidx] : 0.0; }
  double t(size_t tidx, double t0); // 生存時間を設定する
  void setdef()                     // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check(size_t ni);         // オプションをチェックする
};

option::option(const char* c_pname) : vpath(), vpname(), blst(), vbvec(), vcc(CCdef), vdcode("0"), veps(EPSdef), vmethod(methoddif), vmodel(modelexp), vrmax(RMAXdef), tlst(), vtvec()
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "b")            // 偏回帰係数初期値の指定
  {
    if (strsplit(blst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "cc")      // 信頼係数の指定
  {
    vcc = opt[1].empty() ? CCdef : strtodf(opt[1]);
    if (vcc < 0.0 || 1.0 <= vcc) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "d")       // 死亡のコード指定
  { vdcode = opt[1]; }
  else if (opt[0] == "eps")     // 最尤法の収束誤差の指定
  {
    veps = opt[1].empty() ? EPSdef : strtodf(opt[1]);
    if (veps < DBL_EPSILON) err = badopt(vpname, optstr, USAGE, DBL_EPSILON, DBL_MAX);
  }
  else if (opt[0] == "method")  // 計算方法の種類指定
  {
    if (opt[1].empty() || opt[1] == "dif") vmethod = methoddif;
    else if (opt[1] == "del") vmethod = methoddel;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "model")   // 分布モデルの種類指定
  {
    if (opt[1].empty() || opt[1] == "exp") vmodel = modelexp;
    else if (opt[1] == "wei") vmodel = modelwei;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "rmax")    // 最大計算回数指定
  {
    char* ptr = nullptr;
    vrmax = strtol(opt[1].c_str(), &ptr, 0);
    if (vrmax < 0) err = badopt(vpname, optstr, USAGE, 0.0, (double)INT_MAX);
  }
  else if (opt[0] == "t")       // 生存時間指定
  {
    if (strsplit(tlst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this help message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

double option::t(size_t tidx, double t0) // 生存時間を設定する
{
  if (tidx >= vtvec.size()) return 0.0;
  vtvec[tidx] = t0;
  return vtvec[tidx];
}

int option::check(size_t ni)    // オプションをチェックする
{
  if (ni < 3) return disperr(vpname, "At least three items are required !");
  size_t nx = ni - 1;
  size_t np = (vmodel == modelwei) ? nx + 1 : nx;
  if (blst.size() > np) return disperr(vpname, "Too many initial value of the partial regression coefficient !");
  vbvec.resize(np);
  vbvec.assign(np, BLANKf);
  if (blst.size() > 0)
  {
    size_t i = 0;
    for (lstr_citr bpos = blst.begin(); bpos != blst.end(); ++bpos, ++i)
    {
      double fdata = strtodf(*bpos);
      if (fdata == BLANKf) return disperr(vpname, "Wrong initial value->", *bpos);
      vbvec[i] = fdata;
    }
  }
  if (tlst.size() < 1)          // オプション省略時
  {
    vtvec.resize(1);
    vtvec[0] = BLANKf;
  }
  else
  {
    vtvec.resize(tlst.size());
    size_t i = 0;
    for (lstr_citr tpos = tlst.begin(); tpos != tlst.end(); ++tpos, ++i)
    {
      double fdata = strtodf(*tpos);
      if (fdata == BLANKf) return disperr(vpname, "Wrong verification period->", *tpos);
      vtvec[i] = fdata;
    }
  }
  return 0;                     // エラーなし
}

class varclass                  // 変数クラス
{
private:
  int vnx;                       // 共変数の数
  int vnp;                       // パラメータ数
  double lamda0;                 // 初期ハザード
  listlfm lifemlst;              // 生命表リスト
  vector<statv<double>> statvec; // 共変数の基礎統計量ベクトル
  mat<double> smat;              // 積和行列
  listvecd vcvdveclst;           // 交差検証結果リスト
  vecdbl bvec;                   // 偏回帰係数ベクトル
  vecdbl vbvec;                  // 偏回帰係数の分散ベクトル
  vecint nd0vec;                 // 全体の死亡例数ベクトル
  vecint ndt0vec;                // 全体の死亡予測的中例数ベクトル
  vecint na0vec;                 // 全体の生存例数ベクトル
  vecint nat0vec;                // 全体の生存予測的中例数ベクトル
  vecint ndvec;                  // 検証群の死亡例数ベクトル
  vecint ndtvec;                 // 検証群の死亡予測的中例数ベクトル
  vecint navec;                  // 検証群の生存例数ベクトル
  vecint natvec;                 // 検証群の生存予測的中例数ベクトル
  int set_lifemlst(option& opt, ldata& datalst, int vno);
  // 生命表と統計量をセットし、死亡例数を返す
  int cal_pltcv(const option& opt); // パラメトリックモデルを計算する
  void set_initb(vecdbl& b0vec, const option& opt);
  // 重回帰分析による偏回帰係数初期値のセット
  int newton_pltcv(double& ll, const option& opt);
  // Newton法によるパラメトリックモデルの計算(最尤法)
  bool cal_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& bv, const option& opt) const;
  // 傾斜ベクトルgとHessian行列Hを計算する
  bool expdel_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv) const;
  // 指数分布モデルの傾斜ベクトルgとHessian行列Hを差分法で計算する
  double exp_llfun(const vec<double>& hv) const;
  // 指数分布モデルの対数尤度を返す
  bool expdif_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& bv) const;
  // 指数分布モデルの傾斜ベクトルgとHessian行列Hを微分法で計算する
  bool weidel_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv) const;
  // Weibull分布モデルの傾斜ベクトルgとHessian行列Hを差分法で計算する
  double wei_llfun(const vec<double>& hv) const;
  // Weibull分布モデルの対数尤度を返す
  bool weidif_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& bv) const;
  // Weibull分布モデルの傾斜ベクトルgとHessian行列Hを微分法で計算する
  void val_cdata(const option& opt, datac& vdata);
  // 交差群のデータを検証する
  void set_cvlst(const option& opt, int vno);
  // 交差検証結果をリストにセットする
public:
  varclass(const option& opt, size_t ni0); // コンストラクタ
  ~varclass()                   // デストラクタ
  {}
  int cal(option& opt, ldata& datalst); // データを集計して変数をセットする
  int nx() const                        // 共変数の数を返す
  { return vnx; }
  int np() const                // パラメータ数を返す
  { return vnp; }
  const listvecd& cvdveclst() const // 交差検証結果リストを返す
  { return vcvdveclst; }
};

varclass::varclass(const option& opt, size_t ni0) : vnx(ni0 - 1), vnp(0), lamda0(0.0), lifemlst(), statvec(ni0), smat(ni0), vcvdveclst(), bvec(), vbvec(), nd0vec(opt.nt(), 0), ndt0vec(opt.nt(), 0), na0vec(opt.nt(), 0), nat0vec(opt.nt(), 0), ndvec(opt.nt(), 0), ndtvec(opt.nt(), 0), navec(opt.nt(), 0), natvec(opt.nt(), 0)
// コンストラクタ
{
  vnp = (opt.model() == modelwei) ? ni0 : ni0 - 1;
  bvec.resize(vnp);  vbvec.resize(vnp);
  bvec.assign(vnp, 0.0);  vbvec.assign(vnp, 0.0);
}

int varclass::set_lifemlst(option& opt, ldata& datalst, int vno)
// 生命表と統計量をセットし、死亡例数を返す
{
  lifemlst.clear();
  int dim = vnx + 1, nd = 0;
  for (int i = 0; i < dim; ++i) statvec[i].clear();
  smat = 0.0;
  vecdbl fdatavec(dim);
  fdatavec[0] = 1.0;
  int cno = 1;
  double st = 0.0, tmax = 0.0;
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr, ++cno)
  {
    if (cno != vno)             // 検証データ以外を集計
    {
      double t = ditr->fdata(0);
      if (t > tmax) tmax = t;
      vecdbl xvec(vnx - 1);
      statvec[0].sigma(t);
      for (int j = 1; j < vnx; ++j)
      {
        fdatavec[j] = ditr->fdata(j + 1);
        statvec[j].sigma(fdatavec[j]);  xvec[j - 1] = fdatavec[j];
      }
      octype oc = ocalive;      // 生存
      if (ditr->sdata(1) == opt.dcode()) { oc = ocdeath;  ++nd; } // 死亡
      else if (ditr->sdata(1) == BLANKs) oc = ocdropout; // 脱落
      lifem lifemi(cno, t, oc, xvec);
      lifemlst.push_back(lifemi); // 生命表リストに追加する
      if (t > 0.0)                // 重回帰分析用集計
      {
        double lnt = (t > 0.0) ? log(t) : 0.0;
        statvec[dim - 1].sigma(lnt);  fdatavec[dim - 1] = lnt;
        for (int i = 0; i < dim; ++i)
        {
          double x = fdatavec[i];
          for (int j = i; j < dim; ++j) smat.add(i, j, x * fdatavec[j]);
        }
      }
    }
  }
  lamda0 = (statvec[0].sum() > 0.0) ? (double)nd / statvec[0].sum() : 0.0;
  lifemlst.sort();              // 生命表リストをソートする
  int n = lifemlst.size();
  if (n < 1) return -1;         // 生命表データなし
  int i = 0;
  double s = 1.0;
  for (listlfm_itr lpos = lifemlst.begin(); lpos != lifemlst.end(); ++lpos, ++i)
  {
    if (lpos->oc() == ocdeath) s *= (double)(n - i - 1) / (double)(n - i);
    lpos->s(s);                 // 累積生存率をセットする
  }
  if (opt.nt() == 1 && opt.t(0) == BLANKf) opt.t(0, tmax);
  return nd;                    // 死亡例数を返す
}

int varclass::cal_pltcv(const option& opt) // パラメトリックモデルを計算する
{
  int dim = vnx + 1;
  for (int i = 0; i < dim - 1; ++i)
  {
    for (int j = i + 1; j < dim; ++j) smat.assign(j, i, smat(i, j));
  }
  smat.sweep(0);
  vecdbl b0vec(vnp, 0.0);
  set_initb(b0vec, opt);
  bvec = b0vec;
  double ll = 0.0;
  return newton_pltcv(ll, opt);
}

void varclass::set_initb(vecdbl& b0vec, const option& opt)
// 重回帰分析による偏回帰係数初期値のセット
{
  for (int i = 1; i < vnx; ++i) smat.sweep(i);
  const double Beps = DBL_EPSILON * fmax(5.0, smat.maxabse());
  int ks = 0, k = 0;
  if (opt.model() == modelwei)
  {
    ks = k = 1;
    b0vec[0] = 0.1;             // ln(a)の初期値
  }
  b0vec[ks] = log(lamda0);  ++k;
  for (int i = 1; i < vnx; ++i, ++k)
  {
    b0vec[k] = -smat(i, vnx);
    if (fabs(b0vec[k]) < Beps) b0vec[k] = 0.0;
    b0vec[ks] -= b0vec[k] * statvec[i].mean();
  }
  for (int i = 0; i < vnp; ++i)
  {
    if (opt.b(i) != BLANKf) b0vec[i] = opt.b(i);
  }
}

int varclass::newton_pltcv(double& ll, const option& opt)
// Newton法によるパラメトリックモデルの計算(最尤法)
{
  vec<double> bv(vnp), gv(vnp);
  mat<double> hmat(vnp);
  for (int i = 0; i < vnp; ++i) bv.assign(i, bvec[i]);
  double lold = 0.0;
  bool cal = true;
  int nr = 0;
  while (cal && cal_lgh(gv, hmat, ll, bv, opt))
  {
    ++nr;
    hmat.inv();
    if (fabs(ll - lold) < opt.eps() || (opt.rmax() > 0 && nr >= opt.rmax()))
    { cal = false; }
    else
    { lold = ll;  bv -= hmat * gv; }
  }
  if (cal) return -1;           // Newton法発散
  for (int i = 0; i < vnp; ++i) { bvec[i] = bv(i);  vbvec[i] = -hmat(i, i); }
  return nr;                    // 繰り返し回数を返す
}

bool varclass::cal_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& bv, const option& opt) const
// 傾斜ベクトルgとHessian行列Hを計算する
{
  bool ok = true;
  switch (opt.model())
  {
  case modelwei :
    if (opt.method() == methoddel) ok = weidel_lgh(gv, hmat, ll, bv);
    else ok = weidif_lgh(gv, hmat, ll, bv);
    break;
  default :
    if (opt.method() == methoddel) ok = expdel_lgh(gv, hmat, ll, bv);
    else ok = expdif_lgh(gv, hmat, ll, bv);
  }
  return ok;
}

bool varclass::expdel_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv) const
// 指数分布モデルの傾斜ベクトルgとHessian行列Hを差分法で計算する
{
  constexpr double Emax = 308.0; // 指数最大数
  int nh = hv.dim();            // パラメータ数
  for (int i = 0; i < nh; ++i)
  {
    if (isnan(hv(i)) || hv(i) >= Emax) return false; // Newton法発散
  }
  ll = exp_llfun(hv);
  if (isnan(ll) || ll > 0.0 || ll < -DBL_MAX) return false; // Newton法発散
  const double Deps = DBL_EPSILON * fmax(5.0, hv.maxabse());
  constexpr double Delta = 1.0e-5; // 差分法の相対微小変動
  for (int i = 0; i < nh; ++i)
  {
    vec<double> hiv(hv);
    double dhi = (fabs(hv(i)) < Deps) ? Delta : hv(i) * Delta;
    hiv.add(i, dhi);
    double lli = exp_llfun(hiv);
    gv.assign(i, (lli - ll) / dhi);
    for (int j = i; j < nh; ++j)
    {
      vec<double> hjv(hv);
      double dhj = (fabs(hv(j)) < Deps) ? Delta : hv(j) * Delta;
      hjv.add(j, dhj);
      double llj = exp_llfun(hjv);
      vec<double> hijv(hiv);
      hijv.add(j, dhj);
      double llij = exp_llfun(hijv);
      hmat.assign(i, j, (llij - lli - llj + ll) / dhi / dhj);
    }
  }
  for (int i = 0; i < nh - 1; ++i)
  {
    for (int j = i + 1; j < nh; ++j) hmat.assign(j, i, hmat(i, j));
  }
  return true;                  // エラーなし
}

double varclass::exp_llfun(const vec<double>& hv) const
// 指数分布モデルの対数尤度を返す
{
  vec<double> xv(vnp);
  xv.assign(0, 1.0);
  double ll = 0.0;
  for (listlfm_citr litr = lifemlst.begin(); litr != lifemlst.end(); ++litr)
  {
    double ti = litr->t();
    for (int i = 1; i < vnp; ++i) xv.assign(i, litr->x(i - 1));
    double lhi = inp(hv, xv);
   if (litr->oc() == ocdeath) ll += lhi; // 死亡例
   ll += -exp(lhi) * ti;
  }
  return ll;
}

bool varclass::expdif_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& bv) const
// 指数分布モデルの傾斜ベクトルgとHessian行列Hを微分法で計算する
{
  constexpr double Emax = 308.0; // 指数最大値
  double sdlh = 0.0, sth = 0.0;
  vec<double> sdxv(vnp), sthxv(vnp), xv(vnp);
  bool ok = true;
  xv.assign(0, 1.0);
  gv = 0.0;  hmat = 0.0;
  listlfm_citr litr = lifemlst.begin();
  while (ok && litr != lifemlst.end())
  {
    int j = 1;
    for (int i = 1; i < vnp; ++i) xv.assign(i, litr->x(i - 1));
    double lhi = inp(bv, xv);
    if (isnan(lhi) || fabs(lhi) >= Emax) // 発散
    { ok = false; }
    else
    {
      double hi = exp(lhi), ti = litr->t();
      sth += ti * hi;
      for (int j = 0; j < vnp; ++j)
      {
        sthxv.add(j, ti * hi * xv(j));
        for (int k = j; k < vnp; ++k) hmat.add(j, k, -ti * hi * xv(j) * xv(k));
      }
      if (litr->oc() == ocdeath)
      {
        sdlh += lhi;
        for (int j = 0; j < vnp; ++j) sdxv.add(j, xv(j));
      }
      ++litr;
    }
  }
  if (ok == false) return ok;
  ll = sdlh - sth;  gv = sdxv - sthxv;
  for (int i = 0; i < vnp - 1; ++i)
  {
    for (int j = i + 1; j < vnp; ++j) hmat.assign(j, i, hmat(i, j));
  }
  return ok;
}

bool varclass::weidel_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& hv) const
// Weibull分布モデルの傾斜ベクトルgとHessian行列Hを差分法で計算する
{
  constexpr double Emax = 308.0; // 指数最大数
  int nh = hv.dim();            // パラメータ数
  for (int i = 0; i < nh; ++i)
  {
    if (isnan(hv(i)) || hv(i) >= Emax) return false; // Newton法発散
  }
  ll = wei_llfun(hv);
  if (isnan(ll) || ll > 0.0 || ll < -FLT_MAX) return false; // Newton法発散
  const double Deps = DBL_EPSILON * fmax(5.0, hv.maxabse());
  constexpr double Delta = 1.0e-5; // 差分法の相対微小変動
  for (int i = 0; i < nh; ++i)
  {
    vec hiv(hv);
    double dhi = (fabs(hv(i)) < Deps) ? Delta : hv(i) * Delta;
    hiv.add(i, dhi);
    double lli = wei_llfun(hiv);
    gv.assign(i, (lli - ll) / dhi);
    for (int j = i; j < nh; ++j)
    {
      vec hjv(hv);
      double dhj = (fabs(hv(j)) < Deps) ? Delta : hv(j) * Delta;
      hjv.add(j, dhj);
      double llj = wei_llfun(hjv);
      vec hijv(hiv);
      hijv.add(j, dhj);
      double llij = wei_llfun(hijv);
      hmat.assign(i, j, (llij - lli - llj + ll) / dhi / dhj);
    }
  }
  for (int i = 0; i < nh - 1; ++i)
  {
    for (int j = i + 1; j < nh; ++j) hmat.assign(j, i, hmat(i, j));
  }
  return true;                  // エラーなし
}

double varclass::wei_llfun(const vec<double>& hv) const
// Weibull分布モデルの対数尤度を返す
{
  int nxt = vnp - 1;
  vec<double> xv(nxt);
  xv.assign(0, 1.0);
  double ll = 0.0, a = exp(hv(0));
  for (listlfm_citr litr = lifemlst.begin(); litr != lifemlst.end(); ++litr)
  {
    double ti = litr->t();
    if (ti > 0.0)
    {
      for (int i = 1; i < nxt; ++i) xv.assign(i, litr->x(i - 1));
      double lhi = 0.0;
      for (int i = 0; i < nxt; ++i) lhi += hv(i + 1) * xv(i);
      if (litr->oc() == ocdeath) ll += a * lhi + hv(0) + (a - 1.0) * log(ti);
      ll += -pow(exp(lhi) * ti, a);
    }
  }
  return ll;
}

bool varclass::weidif_lgh(vec<double>& gv, mat<double>& hmat, double& ll, const vec<double>& bv) const
// Weibull分布モデルの傾斜ベクトルgとHessian行列Hを微分法で計算する
{
  constexpr double Emax = 308.0; // 指数最大値
  double la = bv(0);
  if (isnan(la) || fabs(la) >= Emax) return false; // 発散
  double a = exp(la);
  int nxt = vnp - 1;
  double sdlh = 0.0, sdlt = 0.0, shta = 0.0, shtalh = 0.0;
  double shtalh2 = 0.0, shtalhlt = 0.0, shtalt = 0.0, shtalt2 = 0.0;
  vec<double> sdxv(nxt), xv(nxt);
  int nd = 0;
  bool ok = true;
  xv.assign(0, 1.0);
  gv = 0.0;  hmat = 0.0;
  listlfm_citr litr = lifemlst.begin();
  while (ok && litr != lifemlst.end())
  {
    double ti = litr->t();
    if (ti > 0.0)
    {
      for (int i = 1; i < vnp - 1; ++i) xv.assign(i, litr->x(i - 1));
      double lhi = 0.0;
      for (int i = 0; i < nxt; ++i) lhi += bv(i + 1) * xv(i);
      if (isnan(lhi) || fabs(lhi) >= Emax) // 発散
      { ok = false; }
      else
      {
        double hi = exp(lhi), lti = log(ti);
        if (litr->oc() == ocdeath) // 死亡例
        {
          ++nd;  sdlh += lhi;  sdlt += lti;
          for (int i = 0; i < nxt; ++i) sdxv.add(i, xv(i));
        }
        double htai = pow(hi * ti, a);
        shta += htai;  shtalh += htai * lhi;
        shtalh2 += htai * lhi * lhi;  shtalhlt += htai * lhi * lti;
        shtalt += htai * lti;  shtalt2 += htai * lti * lti;
        for (int i = 0; i < nxt; ++i)
        {
          gv.add(i + 1, -a * htai * xv(i));
          hmat.add(0, i + 1, -a * a * htai * lhi * xv(i) - a * htai * xv(i) - a * a * htai * lti * xv(i));
        }
        for (int i = 0; i < nxt; ++i)
        {
          for (int j = i; j < nxt; ++j) hmat.add(i + 1, j + 1, -a * a * htai * xv(i) * xv(j));
        }
      }
    }
    ++litr;
  }
  if (ok == false) return ok;
  ll = a * sdlh + (double)nd * la + (a - 1.0) * sdlt - shta;
  gv.assign(0, (double)nd + a * (sdlh + sdlt - shtalh - shtalt));
  hmat.assign(0, 0, a * (sdlh + sdlt - shtalh - a * shtalh2 - 2.0 * a * shtalhlt - shtalt - a * shtalt2));
  for (int i = 0; i < nxt; ++i)
  { gv.add(i + 1, a * sdxv(i));  hmat.add(0, i + 1, a * sdxv(i)); }
  for (int i = 0; i < vnp - 1; ++i)
  {
    for (int j = i + 1; j < vnp; ++j) hmat.assign(j, i, hmat(i, j));
  }
  return ok;
}

void varclass::val_cdata(const option& opt, datac& vdata)
// 交差群のデータを検証する
{
  double llamda0 = log(lamda0);
  int b0 = 0;
  if (opt.model() == modelwei) b0 = 1;
  double llamda = bvec[b0], llamdam = bvec[b0];
  int j = 2;
  for (int i = b0 + 1; i < vnp; ++i, ++j)
  {
    llamda += bvec[i] * vdata.fdata(j);
    llamdam += bvec[i] * statvec[j - 1].mean();
  }
  for (size_t i = 0; i < opt.nt(); ++i)
  {
    int nd = 0, ndt = 0, na = 0 , nat = 0;
    double t = vdata.fdata(0);
    if (t > opt.t(i))           // 生存例
    {
      ++na;
      if (llamda < llamdam) ++nat;
    }
    else if (t == opt.t(i))
    {
      if (vdata.sdata(1) == opt.dcode()) // 死亡例
      {
        ++nd;
        if (llamda >= llamdam) ++ndt;
      }
      else                      // 生存例
      {
        ++na;
        if (llamda < llamdam) ++nat;
      }
    }
    else if (vdata.sdata(1) == opt.dcode()) // 死亡例以外は除外
    {
      ++nd;
      if (llamda >= llamdam) ++ndt;
    }
    ndvec[i] = nd;  ndtvec[i] = ndt;  navec[i] = na;  natvec[i] = nat;
  }
}

void varclass::set_cvlst(const option& opt, int vno)
// 交差検証結果をリストにセットする
{
  double llamda0 = log(lamda0);
  vecdbl cvdvec(3 + vnp + opt.nt() * 6);
  cvdvec[0] = (double)((vno < 1) ? vnp + 2 : vno);
  cvdvec[1] = bvec[0];
  int b0 = 0;
  if (opt.model() == modelwei)
  {
    cvdvec[2] = bvec[1];
    b0 = 1;
  }
  double llamdam = bvec[b0];
  int j = 2;
  for (int i = b0 + 1; i < vnp; ++i, ++j)
  {
    llamdam += bvec[i] * statvec[j - 1].mean();
    cvdvec[i + 1] = bvec[i];
  }
  cvdvec[vnp + 1] = lamda0;
  cvdvec[vnp + 2] = exp(llamdam);
  for (size_t i = 0; i < opt.nt(); ++i)
  {
    int j = vnp + 2 + i * 6;
    cvdvec[j + 1] = (double)((vno > 0) ? ndvec[i] : nd0vec[i]);
    cvdvec[j + 2] = (double)((vno > 0) ? ndtvec[i] : ndt0vec[i]);
    cvdvec[j + 3] = (double)((vno > 0) ? navec[i] : na0vec[i]);
    cvdvec[j + 4] = (double)((vno > 0) ? natvec[i] : nat0vec[i]);
    cvdvec[j + 5] = (double)((vno > 0) ? ndvec[i] + navec[i] : nd0vec[i] + na0vec[i]);
    cvdvec[j + 6] = (double)((vno > 0) ? ndtvec[i] + natvec[i] : ndt0vec[i] + nat0vec[i]);
  }
  vcvdveclst.push_back(cvdvec);
}

int varclass::cal(option& opt, ldata& datalst)
// データを集計して変数をセットする
{
  int nd = 0;                       // 死亡例数
  if ((nd = set_lifemlst(opt, datalst, 0)) > 0)   // 全例計算
  {
    if (cal_pltcv(opt) >= 0)
    {
      for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr)
      {
        val_cdata(opt, *ditr);
        for (size_t i = 0; i < opt.nt(); ++i)
        {
          nd0vec[i] += ndvec[i];  ndt0vec[i] += ndtvec[i];
          na0vec[i] += navec[i];  nat0vec[i] += natvec[i];
        }
      }
      set_cvlst(opt, 0);
    }
  }
  if (nd < 0) return nd;        // 生命表データなし
  int vno = 1;
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr, ++vno)
  {
    if ((nd = set_lifemlst(opt, datalst, vno)) > 0) // 検証群だけ計算
    {
      if (cal_pltcv(opt) >= 0)
      {
        val_cdata(opt, *ditr);
        set_cvlst(opt, vno);
      }
    }
  }
  return (int)(lifemlst.size()); // 生命表の例数を返す
}

class pltcvclass                // pltcvメインクラス
{
private:
  string fname;                 // ファイル名
  void put_dns(ostream& fout, const option& opt, idd& id, const varclass& var) const;
  // 解析結果をデータファイル形式で出力する
public:
  pltcvclass() : fname()        // コンストラクタ
  {}
  ~pltcvclass()                 // デストラクタ
  {}
  int get(const char* c_fname, option& opt, idd& id, ldata& datalst);
  // データを読み込み保存・累積する
  void put(option& opt, idd& id, ldata& datalst) const;
  // 集計結果を出力する
};

void pltcvclass::put_dns(ostream& fout, const option& opt, idd& id, const varclass& var) const
// 解析結果をデータファイル形式で出力する
{
  idd tid(id.title(), 3 + var.np() + opt.nt() * 6);
  tid.assign_item(0, "Number of Parameters/Validation Group Number");
  int j = 1;
  if (opt.model() == modelwei)
  { tid.assign_item(1, "ln(a)");  ++j; }
  tid.assign_item(j, "b0:Intercept");
  ++j;
  for (int i = 2; i <= var.np(); ++i, ++j)
  {
    ostringstream ostr;
    ostr << "b" << j - 1 << ":";
    tid.assign_item(j, ostr.str()+id.item(i));
  }
  tid.assign_item(var.np() + 1, "Overall Hazard");
  tid.assign_item(var.np() + 2, "Mean Hazard");
  for (size_t i = 0; i < opt.nt(); ++i)
  {
    j = var.np() + 2 + i * 6;
    ostringstream ostr;
    ostr << "After " << opt.t(i) << " Period:";
    tid.assign_item(j + 1, ostr.str()+"Deaths");
    tid.assign_item(j + 2, ostr.str()+"Correct Death Prediction");
    tid.assign_item(j + 3, ostr.str()+"Surviving Cases");
    tid.assign_item(j + 4, ostr.str()+"Correct Suviving Prediction");
    tid.assign_item(j + 5, ostr.str()+"Total Cases");
    tid.assign_item(j + 6, ostr.str()+"Total Correct Prediction");
  }
  id.put_title(fout, formtag);
  id.put_stra(fout, formtag);
  tid.put_item(fout, formtag);
  int prec0 = fout.precision(); // 現在の不動小数点精度(有効数字)を保存する
  for (listvecd_citr cvitr = (var.cvdveclst()).begin(); cvitr != (var.cvdveclst()).end(); ++cvitr)
  {
    fout << (*cvitr)[0];
    fout.precision(16);
    for (int i = 1; i <= var.np() + 2; ++i) fout << CHRdelm << (*cvitr)[i];
    fout.precision(prec0);
    for (int i = var.np() + 3; i < (int)(tid.ni()); ++i) fout << CHRdelm << (*cvitr)[i];
    fout << endl;
  }
}

int pltcvclass::get(const char* c_fname, option& opt, idd& id, ldata& datalst)
// データを読み込み保存・累積する
{
  if (c_fname == nullptr) fname.erase();
  else fname = c_fname;
  int nc = fgetdata(fname, id, datalst);
  if (nc < 0) throw runtime_error("Run-time error !");
  if (opt.check(id.ni()) != 0) throw option_error("Run-time option error !");
  nc = (int)id.nc();            // ケース数変更用
  ldata_itr ditr = datalst.begin();
  while (ditr != datalst.end()) // リストの要素を削除するのでforではなくwhile
  {
    bool blank = (ditr->fdata(0) == BLANKf) ? true : false;
    for (size_t j = 2; blank == false && j < id.ni(); ++j)
    {
      if (ditr->fdata(j) == BLANKf) blank = true;
    }
    if (blank)                  // ケースを削除する
    { ditr = datalst.erase(ditr);  --nc; }
    else
    { ++ditr; }
  }
  if (nc < (int)id.nc()) id.nc(nc); // ケース数を減らす
  return nc;
}

void pltcvclass::put(option& opt, idd& id, ldata& datalst) const
// 集計結果を出力する
{
  if (id.nc() < 1) return;
  varclass var(opt, id.ni());   // 変数オブジェクト
  if (var.cal(opt, datalst) > 0) put_dns(cout, opt, id, var);
}

int main(int argc, char* argv[]) // メインルーチン
{
  pltcvclass pltcv;             // メインクラス変数
  option opt(argv[0]);          // オプション
  idd id;                       // データ識別情報
  ldata datalst;                // データリスト
  try
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理フラッグ
    for (int i = 1; i <= argc; ++i)
    {
      if (i >= argc)
      {
        if (fok) pltcv.get(nullptr, opt, id, datalst); // 標準入力から読み込む
      }
      else if (*(argv[i]) == CHRopt)
      { opt.set(argv[i] + 1);  fok = true; }
      else
      { pltcv.get(argv[i], opt, id, datalst);  fok = false; }
    }
    pltcv.put(opt, id, datalst);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "unknown error occured !");
    return 255;
  }
  return 0;
}
